/**************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef DREVO_DREVO_H
#define DREVO_DREVO_H

#include <string>

#include "parametridrevesa.h"
#include "delckidrevesa.h"
#include "pogled.h"
#include "serializatordelckovdrevesa.h"
#include "matrika.h"

class QString;

namespace drevo {
	class ParametriDrevesa;

	/*!
	 * \namespace drevo
	 * \brief imenski prostor za risanje dreves
	 *
	 * Vsa koda za drevesa se nahaja v tem imenskem prostoru.
	 * Razred Drevo zgradi geometrijo na osnovi parametrov proceduralnega modela,
	 * shranjenih v ParametriDrevesa. DelckiDrevesa dejansko rišejo na opengl,
	 * Drevo le dimenzionira delčke. Delčki so veje in listi.
	 *
	 * Pogled definira ostale ne tako pomembne parametre, kako naj izgleda
	 * zgrajena geometrija.
	 *
	 * SerializatorDelckovDrevesa shrani veje in liste drevesa v zunanjo datoteko,
	 * veje oštevilči z enoumnim ID zdolž vseh odsekov veje (za deblo G=0, W=all je ID isti).
	 *
	 * Razredi Tocka, Tocka2D, Matrika, Krivulja, Presek hranijo pomožne strukture za izračunavanje.
	 * Tocka hrani double 3d koordinate točke, Tocka2D double 2d koordinate,
	 * Matrika hrani matriko, Krivulja pa kontrolne točke za risanje grafa/krivulje.
	 * Presek hrani točke za presek veje pri izvozu geometrije v zunanjo datoteko.
	 */


	/*!
	 * \brief izris drevesa po modelu [Zamuda04]
	 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
	 *
	 * Razred na podlagi dobljenih parametrov iz ParametriDrevesa po proceduralnem modelu
	 * rekurzivno zgradi geometrijo drevesa. Veje so označene z G in W indeksoma,
	 * ki sta omejena. Ostala dokumentacija o enačbah je v članku [Zamuda04] in
	 * bolj obširno v diplomskem delu [Zamuda06].
	 */
	class Drevo {
	public:
		/**
			 * Inicializira atribute delckiDrevesa, ZACETEK_LISTOV in lowQuality.
		 * @return
		 */
		Drevo();
		/**
		 * Pobriše GLU resurse.
		 * @return
		 */
		~Drevo();

		/**
		 * Inicializira GL v razredu Pogled: kreiranje tekstur.
		 */
		void createKvadraticnePloskve();
		/**
		 * Shrani ID-je tekstur in ponastavi nastavitve teksture.
		 * @param bark
		 * @param leaf
		 */
		void reloadIdTextures( bool bark, bool leaf );
		/**
		 * Označi, da bomo serializirali drevo:
		 * Označi stanje kot shranjevanje geometrije v datoteko.
		 * @param strFileName
		 */
		void markSerializingToFile( const std::string &strFileName );

		/**
		 * Nariše drevo na podanem položaju s podanimi zasuki.
		 * @param dol_pomik pomik drevesa navzdol
		 * @param levo_pomik pomik drevesa levo
		 * @param z_pomik pomik navznoter
		 * @param x_zasuk zasuk okoli osi x
		 * @param y_zasuk zasuk okoli osi y
		 */
		void drawSingleTreeCentered( double dol_pomik, double levo_pomik, double z_pomik, double x_zasuk, double y_zasuk);
		/**
		 * Nariše drevo v ekosistemu, z upoštevanjem LOD.
		 * @param lod
		 */
		void drawSingleTreeAwayLOD(int lod);

		/**
		 * Parametri drevesa, ki se nastavijo od zunaj. Ob izrisovanju se skopirajo
		 * v ločeno strukturo pd, da ne pride do interference.
		 * \sa pd
		 */
		ParametriDrevesa pdPrepared;

	private:
		/**
		 * Tvori geometrijo drevesa in v razred DelckiDrevesa kliče izrisovanje odsekov vej in listov drevesa.
		 * Metoda kljiče sama sebe rekurzivno in tvori odseke vej, dokljer stZil ne pride do 1, ko nariše liste.
		 * Dolžine vej so določene s samopodobnostmi koeficienti za karakteristično višino.
		 * Sproti se vodijo tudi matrike, ki hranijo kompozicije dosedanjih zasukov ali pa tudi premikov:
		 * te služijo preslikavam iz fraktalnega prostora nazaj v osnovni absolutni evklidski merski prostor.
		 * @param stZil število žil v trenuti osnovni veji
		 * @param G indeks odseka veje, ki se poveča za vsako novo stransko podvejo
		 * @param W indeks odseka veje, ki se poveča za vsako nov odsek vej
		 * @param karakteristicnaVisina relativno razmerje višine trenutne osnovne veje glede na prejšnjo osnovno vejo
		 * @param aktivnaVisina absolutna višina trenutne osnovne veje
		 * @param modelMatrixInverse matrika kompozicij dosedanjih zasukov za gradnjo drevesa (brez zibanja v vetru)
		 * @param modelMatrixInversePostGrowthEffects matrika kompozicij dosedanjih zasukov, z upoštevanjem zibanja v vetru
		 * @param presekSpodaj presek spodaj
			 * @param idVeje zaporedna številka prave veje po G, ne le odseka (za združevanje pri branju CRS)
		 */
		void drevca( int stZil, int G, int W, double karakteristicnaVisina, double aktivnaVisina, Matrika modelMatrixInverse, Matrika modelMatrixInversePostGrowthEffects, Presek *presekSpodaj, int idVeje );
		/**
		 * Aplicira gravimorfizem na vejo: tvori gravimorfizemVektor in gravimorfizemAngle za trenutni odsek veje.
		 * @param G indeks odseka veje, ki se poveča za vsako novo stransko podvejo
		 * @param W indeks odseka veje, ki se poveča za vsako nov odsek vej
		 * @param modelMatrixInverse matrika kompozicij dosedanjih zasukov za gradnjo drevesa (brez zibanja v vetru)
		 * @param gravimorfizemAngle izračunan kot zasuka zaradi gravimorfizma za trenutno vejo
		 * @param gravimorfizemVektor izračunan vektor, okoli katerega naj se veja v svojem koordinatnem sistemu zasuče
		 */
		void aplicirajGravmorfizem( int G, int W, Matrika &modelMatrixInverse, double &gravimorfizemAngle, Tocka& gravimorfizemVektor );
		/**
		 * Pripravi inverzno matriko transformacijam v drevesu:
		 * pripravi inverzno matriko iz apliciranih zasukov, tako da zmnoži transformacije v obratnem vrstnem redu.
		 * @param modelMatrixInverse matrika kompozicij dosedanjih zasukov za gradnjo drevesa (brez zibanja v vetru)
		 * @param gravimorfizemAngle kot zasuka zaradi gravimorfizma za trenutno vejo
		 * @param gravimorfizemVektor vektor, okoli katerega naj se veja v svojem koordinatnem sistemu zasuče
		 * @param gravicentralizem kot zasuka okoli centralne osi debla
		 * @param kotVejitve kot vejitve veje stran od osnovne veje
		 * @param windAngle kot zasuka zaradi pihanja vetra za trenutno vejo
		 * @param windVektor vektor, okoli katerega zasučemo zaradi pihanja vetra za trenutno vejo, v svojem koordinatnem sistemu
		 */
		void priraviInverznoMatriko( Matrika &modelMatrixInverse, double gravimorfizemAngle, Tocka &gravimorfizemVektor,
		                             double gravicentralizem, double kotVejitve, double windAngle, Tocka &windVektor );
		/**
		 * Doda vejo v geometrični model, tako da kliče metodo v DelckiDrevesa.
		 * @param dejanskaVisinaTrenutneVeje višina odseka veje, ki bo narisana
		 * @param sirinaSpodaj širina spodnjega oboda odseka veje, ki bo narisana
		 * @param sirinaZgoraj širina zgornjega oboda odseka veje, ki bo narisana
		 * @param presekSpodaj točke preseka spodaj, narisanega odseka veje
		 * @param idVeje zaporedna številka prave veje po G, ne le odseka (za združevanje pri branju CRS)
		 */
		void dodajVejo( double dejanskaVisinaTrenutneVeje, double sirinaSpodaj, double sirinaZgoraj, Presek *presekSpodaj, int idVeje );
		/**
		 * Žile v osnovni veji razdeli v žile v glavni podveji in stranski podveji.
		 * @param G indeks odseka veje, ki se poveča za vsako novo stransko podvejo
		 * @param W indeks odseka veje, ki se poveča za vsako nov odsek vej
		 * @param stZil število žil v osnovni veji drevesa
		 * @param stZilGlavno izračunano število žil v glavni podveji
		 * @param stZilStransko izračunano število žil v stranski podveji
		 * @param kotVejitveGlavno izračunan kot vejitve glavne podveje
		 * @param kotVejitveStransko izračunan kot vejitve stranske podveje
		 */
		void deliVejo( int G, int W, int stZil, int &stZilGlavno, int &stZilStransko, double &kotVejitveGlavno, double &kotVejitveStransko );
		/**
		 * Izračuna dimenzije veje, tako da jo ustrezno skalira in ustrezno omeji.
		 * Velike črke=karakteristična, male črke=aktivna dolžina.
		 *
		 * @param G indeks odseka veje, ki se poveča za vsako novo stransko podvejo
		 * @param W indeks odseka veje, ki se poveča za vsako nov odsek vej
		 * @param stZil število žil v osnovni veji drevesa
		 * @param stZilGlavno izračunano število žil v glavni podveji
		 * @param stZilStransko izračunano število žil v stranski podveji
		 * @param L0 karakteristična dolžina trenutne veje
		 * @param sirinaSpodaj širina spodnjega oboda odseka veje, ki bo narisana
		 * @param sirinaZgoraj širina zgornjega oboda odseka veje, ki bo narisana
		 * @param L1 karakteristična dolžina za glavno podvejo (MAJOR)
		 * @param l1 aktivna dolžina za glavno podvejo (MAJOR)
		 * @param L2 karakteristična dolžina za stransko podvejo (MINOR)
		 * @param l2 aktivna dolžina za stransko podvejo (MINOR)
		 */
		void izracunajDimenzijeVeje( int G, int W, double stZil, double stZilGlavno, double stZilStransko, double L0,
		                             double &sirinaSpodaj, double &sirinaZgoraj,
		                             double &L1, double &l1, double &L2, double &l2 );
		/**
		 * Odsek veje zasuče za zibanje v vetru in pihanje vetra v določeni smeri.
		 * @param stZil število žil v veji drevesa
		 * @param visinaVeje višina odseka veje
		 * @param modelMatrixInversePostGrowthEffects matrika kompozicij dosedanjih zasukov, z upoštevanjem zibanja v vetru
		 * @param windAngle kot zasuka zaradi vetra
		 * @param windVektor vektor, okoli katerega zasučemo zaradi vetra, v koordinatnem sistemu trenutne veje
		 */
		void aplicirajWindSway( int stZil, double visinaVeje, Matrika &modelMatrixInversePostGrowthEffects, double &windAngle, Tocka &windVektor );
		/**
		 * Izračuna vektor zasuka zaradi vetra, glede na trenutni položaj (z inverzno matriko).
		 * S pomočjo inverzne matrike translacij, ki upošteva že poprej apliciran efekt
		 * zibanja v vetru izračuna vektor, okoli katerega sučemo veje zaradi vetra.
		 * @param modelMatrixInversePostGrowthEffects matrika kompozicij dosedanjih zasukov, z upoštevanjem zibanja v vetru
		 * @param vektorZasuka izračunan vektor zasuka
		 */
		void izracunajWindVektorZasuka( Matrika &modelMatrixInversePostGrowthEffects, Tocka& vektorZasuka );

		/**
		 * Določi, ali naj se tvorba odsekov veje tu prekine zaradi omejitev G in W, list se več ne nariše.
		 * Upoteva omejitve drevesa (t. vejitev, t. žil v veji) in vrne kretnico o nadaljnem razvoju.
		 * @param G indeks odseka veje, ki se poveča za vsako novo stransko podvejo
		 * @param W indeks odseka veje, ki se poveča za vsako nov odsek vej
		 * @param stZil število žil v osnovni veji drevesa
		 * @return ali naj na tem nivuju več ne dodam nič geometrije
		 */
		bool constraint( int G, int W, int stZil );
		/**
		 * Določi barvo trenutnemu odseku veje za nivo G,W pri obarvanju debla drevesa.
		 * @param G indeks odseka veje, ki se poveča za vsako novo stransko podvejo
		 * @param W indeks odseka veje, ki se poveča za vsako nov odsek vej
		 * @param colorMonopodial ali naj pobarvam vzdolž veje
		 */
		void dolociBarvo( int G, int W, bool colorMonopodial );
		/**
		 * Popravi rotacijo za phyllotaxis v preseku, da se točke ujemajo ob izvažanju CRS.
		 * @param presekSpodajNovi točke v preseku trenutne veje
		 */
		void popraviPhyllotaxis( Presek &presekSpodajNovi );
		/**
		 * Nazaj rotira za kot s pomočjo openGL.
		 * Rotira izbrano točko okoli središča za določen kot in vrne rotirano točko.
		 * @param sredisce središče, od katerega je točka
		 * @param tocka izbrana točka
		 * @param kot določen kot
		 * @param windAngle kot zasuka zaradi vetra
		 * @param windVektor vektor, okoli katerega zasučemo zaradi vetra, v koordinatnem sistemu trenutne veje
		 */
		void rotirajTocko( Tocka &sredisce, Tocka &tocka, double kot, double &windAngle, Tocka &windVektor );
		/**
		 * Narie določen tip listov na trenutni veji (vejah, ki imajo le 1 žilo).
		 * @param dejanskaVisinaTrenutneVeje absolutna dolžina trenutne veje
		 * @param modelMatrixInversePostGrowthEffects matrika kompozicij dosedanjih zasukov, z upoštevanjem zibanja v vetru
		 * @param idVeje zaporedna številka prave veje po G, ne le odseka (za združevanje pri branju CRS)
		 */
		void narisiListe( double dejanskaVisinaTrenutneVeje, Matrika &modelMatrixInversePostGrowthEffects, int idVeje );

		/**
		 * Priskrbi parametre za novo drevo:
		 * iz pdPrepared parametre prekopira v pd in tako drevo pripravi za upodabljanje.
		 */
		void postaviPrivzeteLastnostiDrevesa();
		/**
		 * Upodobi prikaz v glavnem oknu z izbranimi parametri, ki so shranjeni v pogledu.
		 */
		void upodobiPrikaz( );

		///ATRUBUTI:
	private:
		/**
		 * Število žil najmanj potrebnih, da namesto veje narišemo že liste (vrednost: 1).
		 */
		const int ZACETEK_LISTOV;
		/**
		 * Parametri za proceduralni model drevesa, ki tvori geometrijo.
		 */
		ParametriDrevesa pd;
		/**
		 * Objekt za izrisovanje elementov drevesa: veje in listi.
		 */
		DelckiDrevesa delckiDrevesa;
		/**
		 * Ime datoteko, v katero naj se shranijo ogljišče za CRS geometrijo.
		 */
		std::string fileNameExportCRS;
		/**
		 * Zaporedna številka prave veje po G, ne le odseka (za združevanje pri branju CRS).
		 */
		int idVejaNext;

		/**
		 * Za posebno obarvanje podstebel.
		 */
		float barvaStebla[ 3 ];

		//TODO: mora res biti public?
	public:
		/**
		 * Določa pogled na grajeno geometrijo: izris wireframe, starost animiranega drevesa in drugo.
		 */
		Pogled pogled;
		/**
		 * Objekti za serializacijo drevesa v datoteko.
		 */
		SerializatorDelckovDrevesa serializatorDelckovDrevesa;
	};
}
#endif
