/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef DREVO_PARAMETRIDREVESA_H
#define DREVO_PARAMETRIDREVESA_H

#define MAX_GRA 15
#define MAX_WEI 50

#include <string>
#include <fstream>

#include "krivulja.h"

namespace drevo {
	/**
	 * \brief Parametri drevesa, ki določajo geometrijo
	 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
	*/
	class ParametriDrevesa {
	public:
		/**
		 * Ustvari nove parametre drevesa in kliče nastavitev privzetih parametrov.
		 * @return
		 */
		ParametriDrevesa();
		~ParametriDrevesa();

		/**
		 * Število žil v drevesu.
		 */
		int stVsehZil;

		/**
		 * Dolžina prvega odseka osnovnega debla.
		 */
		double dolzinaDebla;
		/**
		 * Širina stebla za eno žilo.
		 */
		float sirinaSteblaNaZilo;

		/**
		 * Kot rotacije filotakse: kot rotacije okoli centralne osi veje
		 */
		double phyllotaxisAngle;

		/**
		 * Odstotek žil, ki naj gre v glavno podvejo, odvisno od indeksov G in W.
		 */
		double verjetnostVejitve[ MAX_GRA ][ MAX_WEI ];
		/**
		 * Kot vejitve med stranskima vejama, odvisno od indeksov G in W.
		 */
		double kotVejitve[ MAX_GRA ][ MAX_WEI ];

		/**
		 * Maksimalna karakteristična dolžina podveje, odvisno od indeksov G in W.
		 */
		double charMax[ MAX_GRA ][ MAX_WEI ];
		/**
		 * Minimalna karakteristična dolžina podveje, odvisno od indeksov G in W.
		 */
		double charMin[ MAX_GRA ][ MAX_WEI ];

		/**
		 * Faktor, za katerega skaliramo dolžino veje za izračun njene dejanske dolžine,
		 * odvisno od indeksov G in W.
		 */
		double lengthScaling[ MAX_GRA ][ MAX_WEI ];

		/**
		 * Odstotek gravicentralizma: pomika proti centru debla zaradi izravnavanja krošnje.
		 */
		double graviCentralizem;
		/**
		 * Vpliv gravitacije na povešenost vej, odvisno od indeksov G in W.
		 */
		double graviMorfizem[ MAX_GRA ][ MAX_WEI ];

		/**
		 * Ali ima drevo liste.
		 */
		bool listi;
		/**
		 * Velikost listov.
		 */
		float listSize;
		/**
		 * Gostota listov: število listov na eni veji.
		 */
		int gostotaListov;
		/**
		 * Tip razporeditve listov na veji: 5 tipov.
		 */
		int tipListov;

		/**
		 * Hitrost vetra, ki piha v drevo.
		 */
		double windSpeed;
		/**
		 * Hitrost vetra, ki piha v drevo.
		 */
		double windSpeedNodirect;

		/**
		 * Ime datoteke za nalaganje teksture za skorjo.
		 */
		std::string barkTextureFileName;
		/**
		 * Ime datoteke za nalaganje teksture za liste.
		 */
		std::string leafTextureFileName;

	public:
		/**
		 * Tvori privzete parametre drevesa: bukev.
		 */
		void defaultTreeParams();
		/**
		 * Naloži parametre drevesa iz datoteke.
		 * @param fileName ime datoteke
		 * @return uspešnost nalaganja
		 */
		bool loadParameters(std::string fileName);
		/**
		 * Shrani parametre drevesa v datoteko.
		 * @param fileName ime datoteke
		 * @return uspešnost shranjevanja
		 */
		bool saveParameters(std::string fileName);

	public:
		/**
		 * Tip razporeditve listov na veji.
		 */
		enum tipParametra {STRAND_DISTRIBUTION, BRANCH_ANGLES_DISTRIBUTION,
		                   MIN_CHARACTERISTIC_LENGTH, MAX_CHARACTERISTIC_LENGTH,
		                   LENGTH_SCALING, GRAVIMORPHISM_IMPACT};

		/**
		 * Kontrolne točke grafa za pripadajočo interpolirano polje verjetnostVejitve.
		 */
		Krivulja verjetnostiVejitveG;
		/**
		 * Kontrolne točke grafa za pripadajočo interpolirano polje verjetnostVejitve.
		 */
		Krivulja verjetnostiVejitveW;

		/**
		 * Kontrolne točke grafa za pripadajočo interpolirano polje kotVejitve.
		 */
		Krivulja kotVejitveG;
		/**
		 * Kontrolne točke grafa za pripadajočo interpolirano polje kotVejitve.
		 */
		Krivulja kotVejitveW;

		/**
		 * Kontrolne točke grafa za pripadajočo interpolirano polje minCharacteristicLength.
		 */
		Krivulja minCharacteristicLengthG;
		/**
		 * Kontrolne točke grafa za pripadajočo interpolirano polje minCharacteristicLength.
		 */
		Krivulja minCharacteristicLengthW;
		/**
		 * Kontrolne točke grafa za pripadajočo interpolirano polje maxCharacteristicLength.
		 */
		Krivulja maxCharacteristicLengthG;
		/**
		 * Kontrolne točke grafa za pripadajočo interpolirano polje maxCharacteristicLength.
		 */
		Krivulja maxCharacteristicLengthW;

		/**
		 * Kontrolne točke grafa za pripadajočo interpolirano polje lengthScaling.
		 */
		Krivulja lengthScalingG;
		/**
		 * Kontrolne točke grafa za pripadajočo interpolirano polje lengthScaling.
		 */
		Krivulja lengthScalingW;

		/**
		 * Kontrolne točke grafa za pripadajočo interpolirano polje gravimorphismImpact.
		 */
		Krivulja gravimorphismImpactG;
		/**
		 * Kontrolne točke grafa za pripadajočo interpolirano polje gravimorphismImpact.
		 */
		Krivulja gravimorphismImpactW;
		/**
		 * Interpolira krivuljo parametra in jo shrani v pripadajočo polje parametrov modela.
		 * @param idParametra
		 */
		void preracunajParametra( int idParametra );
		/**
		 * Pomožna funkcija za maksimum.
		 * @param a a
		 * @param b b
		 * @return večji od a in b
		 */
		static double max2( double a, double b ) {
			return a > b ? a : b;
		}
		/**
		 * Pomožna funkcija za minimum.
		 * @param a a
		 * @param b b
		 * @return manjši od a in b
		 */
		static double min2( double a, double b ) {
			return a < b ? a : b;
		}
	private:
		/**
		 * Preračuna (interpolira) vse vektorske parametre,
		 * vse z grafom relativno podane parametre na dejanske parametre glede na G, W.
		 */
		void recalculateRelativeParams();
		/**
		 * Interpolira vrednost pri x na krivulji.
		 * Izračuna interpolirano vrednost na krivulji, s koordinato x enako parametru "x".
		 * @param x
		 * @param parametri
		 * @return
		 */
		double interpoliraj( double x, Krivulja *parametri );

		/**
		 * Shrani kontrolne točke izbrane krivulje v izhodno datoteko.
		 * @param f izhodna datoteka
		 * @param pk izbrana krivulja
		 */
		void shraniParameter(std::ofstream &f, const Krivulja &pk);
		/**
		 * Naloži kontrolne točke izbrane krivulje iz vhodne datoteke.
		 * @param f vhodna datoteka
		 * @param pk izbrana krivulja
		 */
		void naloziParameter(std::ifstream &f, Krivulja &pk);
	};
}
#endif
