/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <QGLWidget>
#include <QtOpenGL>

#include "ecosystemrender.h"
#include "terrain.h"
#include "terraindata.h"
#include "objectculling.h"
#include "levelofdetail.h"
#include "vrstarastlin.h"
#include "rastlina.h"
#include "ecosystemsimulation.h"
#include "terrainmarkings.h"
#include "entitiesstatistics.h"
#include "sky.h"

#include "drevo.h"
#include "camera.h"
#include "matrika.h"
#include <sstream>

#include <iostream>

using namespace ecosystem::render;

EcosystemRender::EcosystemRender( const QGLContext *qglContext, QGLWidget *qGLWidget ) : qGLWidget( qGLWidget ) {
	/// nastavimo skalarje in kretnice za nastavitve pogleda
	simulationStepsPerRenderedFrame = 1;
	simulationStepsUntilRender = simulationStepsPerRenderedFrame;
	autoSaveSimulationStepNr = -1;
	checkingForGlErrors=false;
	singleRender = false;
	lightingEnabled = true;
	showFPS = true;
	wireframe = false;
	showNeighborHood = false;
	doNotDrawTrees = false;
	doNotRefreshDisplay = false;
	displayTerrainNormals = false;
	displayTerrainInclination = false;
	displayTerrainMoisture = false;
	displayTerrainSunniness = false;
	displayTerrainWindiness = false;
	displayLandMarkings = false;
	displaySun = false;
	displaySky = true;

	filenameDetail = ":/gui/textures/detail.png";
	filenameSky1 = ":/gui/textures/sky_n.png";
	filenameSky2 = ":/gui/textures/sky_s.png";
	filenameSky3 = ":/gui/textures/sky_e.png";
	filenameSky4 = ":/gui/textures/sky_w.png";
	filenameSky5 = ":/gui/textures/sky_u.png";
	filenameSky6 = ":/gui/textures/sky_d.png";

	/// nastavimo interni model
	initModelObjects();

	/// nastavimo objekte za izrisovanje scene
	camera = new Camera( this, terrainData );
	terrainMarkings = new TerrainMarkings( terrainData );
	sky = new Sky( terrainData->dimen );
	terrain = new Terrain( qglContext, terrainData );

	/// inicializiramo teksture za drevo, teren in obzorje
	/// heighmap terena tu mora biti že naložen, saj bomo generirali rastersko sliko za njegovo teksturo,
	/// odvidno od višin terena
	initAllTextures( qGLWidget );
}

EcosystemRender::~EcosystemRender() {
	delete terrain;
	delete sky;
	delete terrainData;
	delete terrainMarkings;

	int sz = ( int ) izglediDrevesnihVrst.size();
	for ( int i = 0; i < sz; i++ ) {
		/// pobrišemo teksture drevesa
		GLuint idBark = izglediDrevesnihVrst[ i ] ->pogled.textureNames[ 0 ];
		GLuint idLeaf = izglediDrevesnihVrst[ i ] ->pogled.textureNames[ 1 ];
		glDeleteTextures( 1, &idBark );
		glDeleteTextures( 1, &idLeaf );

		delete izglediDrevesnihVrst[ i ];
	}
}

void EcosystemRender::initModelObjects() {
	///nastavimo podatke simulacije
	terrainData = new ecosystem::data::CTerrainData();
	ecosystemSimulation = new CEcosystemSimulation ();
	ecosystemSimulation->setTerrainData( *terrainData );
	izbranoDrevo = 0;
	char * imenaDreves[] = {"bin/zbirka-4/grm5.tree.xml",
	                        "bin/zbirka-4/Vrba Zalujka (Salix Babylonica).tree.xml",
	                        "bin/zbirka-4/javorVelikZgorajGost.tree.xml",
	                        //"bin/zbirka-4/trava.tree.xml",
	                        "bin/zbirka-4/smrekaMini.tree.xml",
	                        "bin/zbirka-4/manjseDefault100_vecjiListi.tree.xml",
	                        ""};
	for ( int i = 0; i < 6; i++ ) {
		drevo::Drevo *novoDrevo = new drevo::Drevo;
		izbranoDrevo = novoDrevo;
		novoDrevo->createKvadraticnePloskve();
		izbranoDrevo->pdPrepared.loadParameters( imenaDreves[ i ] );
		//TODO:beri iz datoteke konfiguracijo ekosistema
		//novoDrevo->pdPrepared.loadParameters("zbirka-1/default.tree");

		izglediDrevesnihVrst.push_back( novoDrevo );
	}
	indexIzbranegaDrevesa = 5;
	izbranoDrevo = izglediDrevesnihVrst[ indexIzbranegaDrevesa ];

	/// poženemo simulacije lastnosti terena že na začetku..
	if ( !terrainData->isDataReady() )
		terrainData->buildTerrainData();
}

void EcosystemRender::initAllTextures( QGLWidget *qGLWidget ) {
	reloadAllTreeTextures();

	/// SKY:
	sendSkyTexturesToSkyRender();

	/// TERRAIN:

	///pobrišemo teksture za terrain
	GLuint idTexturaTeren = 0;
	GLuint idTexturaDetail = 0;
	terrain->getTextureNames( idTexturaTeren, idTexturaDetail );
	if ( idTexturaTeren )
		qGLWidget->deleteTexture( idTexturaTeren );
	if ( idTexturaDetail )
		qGLWidget->deleteTexture( idTexturaDetail );

	///teren zgenerira CHeighmapFactory in ga shrani v CTerrainData:
	//QString filenameTeren = ":/gui/textures/terrain.png";
	QImage &teren = *terrainData->ortofotoTexture;
	idTexturaTeren = qGLWidget->bindTexture( teren );

	///teksturo za detajl naložimo iz datoteke, fiksno
	QImage detail( filenameDetail );
	idTexturaDetail = qGLWidget->bindTexture( detail );

	terrain->setTextureNames( idTexturaTeren, idTexturaDetail );

	/// starega vertex locka ne pobrišamo, ustvarimo novega za ta model
	terrain->initGLContexMultitextureAndArrays();
}

void EcosystemRender::reloadAllTreeTextures() {
	drevo::Drevo * izbranoDrevoBkp = izbranoDrevo;
	for ( int i = 0; i < 6; i++ ) {
		izbranoDrevo = izglediDrevesnihVrst[ i ];
		reloadCurrentTreeTextures();
	}
	izbranoDrevo = izbranoDrevoBkp;
}

void EcosystemRender::sendSkyTexturesToSkyRender() {
	/// pobrišemo teksture za sky -- not.
	GLuint * skyTexNames = sky->getTextureNames();
	for ( int i = 0; i < 6; i++ )
		if ( skyTexNames[ i ] )
			qGLWidget->deleteTexture( skyTexNames[ i ] );

	///kreiramo teksture za sky
	QImage sky1( filenameSky1 ), sky2( filenameSky2 ), sky3( filenameSky3 ), sky4( filenameSky4 ),
	sky5( filenameSky5 ), sky6( filenameSky6 );
	GLuint idTexturaSky1 = qGLWidget->bindTexture( sky1 );
	GLuint idTexturaSky2 = qGLWidget->bindTexture( sky2 );
	GLuint idTexturaSky3 = qGLWidget->bindTexture( sky3 );
	GLuint idTexturaSky4 = qGLWidget->bindTexture( sky4 );
	GLuint idTexturaSky5 = qGLWidget->bindTexture( sky5 );
	GLuint idTexturaSky6 = qGLWidget->bindTexture( sky6 );
	sky->setTextureNames( idTexturaSky1, idTexturaSky2, idTexturaSky3, idTexturaSky4, idTexturaSky5, idTexturaSky6 );
}

void EcosystemRender::reloadSkyTextures( const QString &filenameSky1,
        const QString &filenameSky2,
        const QString &filenameSky3,
        const QString &filenameSky4,
        const QString &filenameSky5,
        const QString &filenameSky6 ) {
	this->filenameSky1 = filenameSky1;
	this->filenameSky2 = filenameSky2;
	this->filenameSky3 = filenameSky3;
	this->filenameSky4 = filenameSky4;
	this->filenameSky5 = filenameSky5;
	this->filenameSky6 = filenameSky6;

	sendSkyTexturesToSkyRender();
	emit modelDataChanged();
}

void EcosystemRender::getSkyTextureFileNames( QString &filenameSky1,
        QString &filenameSky2,
        QString &filenameSky3,
        QString &filenameSky4,
        QString &filenameSky5,
        QString &filenameSky6 ) {
	filenameSky1 = this->filenameSky1;
	filenameSky2 = this->filenameSky2;
	filenameSky3 = this->filenameSky3;
	filenameSky4 = this->filenameSky4;
	filenameSky5 = this->filenameSky5;
	filenameSky6 = this->filenameSky6;
}


drevo::Drevo* EcosystemRender::getIzbranoDrevo() {
	return izbranoDrevo;
}

Camera *EcosystemRender::getCurrentCamera() {
	return camera;
}

ecosystem::statistics::CEntitiesStatistics* EcosystemRender::getEntitiesStatistics() {
	return ecosystemSimulation->getEntitiesStatistics();
}

void EcosystemRender::reloadDMR( const std::string &fileName ) {
	std::string copy = fileName;
	char * name = ( char* ) copy.c_str();
	terrainData->reloadDMR( name );
}

const ecosystem::data::CTerrainData * EcosystemRender::getTerrainData() const {
	return terrainData;
}

void EcosystemRender::selectNextTreeSpecies() {
	drevo::Drevo * staroDrevo = izbranoDrevo;

	///preklopimo na naslednje drevo
	indexIzbranegaDrevesa++;
	if ( indexIzbranegaDrevesa >= ( int ) izglediDrevesnihVrst.size() )
		indexIzbranegaDrevesa = 0;
	izbranoDrevo = izglediDrevesnihVrst[ indexIzbranegaDrevesa ];

	///za novo drevo ustavimo animacije
	izbranoDrevo->pogled.animateOrbitAround = staroDrevo->pogled.animateOrbitAround;
	izbranoDrevo->pogled.animateWindSway = staroDrevo->pogled.animateWindSway;
	izbranoDrevo->pogled.animateWindSway = staroDrevo->pogled.animateWindSway;

	///za staro ustavimo animacije
	staroDrevo->pogled.animateOrbitAround = false;
	staroDrevo->pogled.animateWindSway = false;
	staroDrevo->pogled.animateWindSway = false;

	emit modelDataChanged();
}

int EcosystemRender::isSimulationRedyAndRunning() {
	return ecosystemSimulation->simulationRunning;
}

void EcosystemRender::actionEnlarge_leave_size() {
	izbranoDrevo->pdPrepared.listSize *= 1.1f;
	emit modelDataChanged();
}

void EcosystemRender::actionReduce_leave_size() {
	izbranoDrevo->pdPrepared.listSize *= 0.91f;
	simulationStepsUntilRender--;
	if ( simulationStepsUntilRender <= 0 ) {
		simulationStepsUntilRender = simulationStepsPerRenderedFrame;
		emit modelDataChanged();
	}
}

void EcosystemRender::actionRender_more_often_skip_less_frames() {
	simulationStepsPerRenderedFrame--;
	if ( simulationStepsPerRenderedFrame <= 0 )
		simulationStepsPerRenderedFrame = 1;
}

void EcosystemRender::actionRender_less_often_skip_more_frames() {
	simulationStepsPerRenderedFrame++;
}

void EcosystemRender::actionGrow_tree() {
	izbranoDrevo->pogled.starostDrevesa = 0;
	izbranoDrevo->pogled.animateGrow = !izbranoDrevo->pogled.animateGrow;
	emit modelDataChanged();
	emit startTreeAnimation();
}

void EcosystemRender::actionSpin_tree() {
	izbranoDrevo->pogled.animateOrbitAround = !izbranoDrevo->pogled.animateOrbitAround;
	emit modelDataChanged();
	emit startTreeAnimation();
}

void EcosystemRender::actionSway_tree_in_wind() {
	izbranoDrevo->pogled.animateWindSway = !izbranoDrevo->pogled.animateWindSway;
	emit modelDataChanged();
	emit startTreeAnimation();
}

void EcosystemRender::setLupaMinX( int val ) {
	terrainData->lupaXmin = val;
	terrainData->buildNonwaterIndexes();
}

void EcosystemRender::setLupaMaxX( int val ) {
	terrainData->lupaXmax = val;
	terrainData->buildNonwaterIndexes();
}

void EcosystemRender::setLupaMinY( int val ) {
	terrainData->lupaYmin = val;
	terrainData->buildNonwaterIndexes();
}

void EcosystemRender::setLupaMaxY( int val ) {
	terrainData->lupaYmax = val;
	terrainData->buildNonwaterIndexes();
}

int EcosystemRender::getLupaMinX() {
	return terrainData->lupaXmin;
}

int EcosystemRender::getLupaMaxX() {
	return terrainData->lupaXmax;
}

int EcosystemRender::getLupaMinY() {
	return terrainData->lupaYmin;
}

int EcosystemRender::getLupaMaxY() {
	return terrainData->lupaXmax;
}


void EcosystemRender::setLightPosition() {
	if ( !singleRender ) {
		GLfloat light_position[] = {GL_DIFFUSE, 0.0, 5750.0, 0.0}; /*0,0,0,1*/
		glLightfv( GL_LIGHT0, GL_POSITION, light_position );
		light_position[ 1 ] += 25 * 100;
		light_position[ 3 ] += 25 * 100;
		glLightfv( GL_LIGHT0 + 1, GL_POSITION, light_position );
		light_position[ 1 ] = 1000;
		light_position[ 3 ] = 715;
		glLightfv( GL_LIGHT0 + 2, GL_POSITION, light_position );
	}
}

void EcosystemRender::simulateNextStepAndUpdateGL() {
	ecosystemSimulation->simulateNextStep();
	std::cout <<getEntitiesStatistics()->getLogText()<<std::endl;

	emit nrPlantsLivingChanged();
	emit modelDataChanged();
	updateAutoSave();
}

void EcosystemRender::reloadCurrentTreeTextures() {
	QString filenameBark( izbranoDrevo->pdPrepared.barkTextureFileName.c_str() );
	QString filenameLeaf( izbranoDrevo->pdPrepared.leafTextureFileName.c_str() );

	filenameBark.replace( '\\', '/' );
	filenameLeaf.replace( '\\', '/' );

	QImage bark, leaf;
	if ( !bark.load( filenameBark ) ) {
		( void ) QMessageBox::information( 0, tr( "Error" ),
		                                   tr( "Could not open bark texture file." ), QMessageBox::Ok );
		return ;
	}
	if ( !leaf.load( filenameLeaf ) ) {
		( void ) QMessageBox::information( 0, tr( "Error" ),
		                                   tr( "Could not open bark texture file." ), QMessageBox::Ok );
		return ;
	}
	///zamenjamo belo barvo za transparentno:
	makeTransparent( &bark );
	makeTransparent( &leaf );

	GLuint &idBark = izbranoDrevo->pogled.textureNames[ 0 ];
	GLuint &idLeaf = izbranoDrevo->pogled.textureNames[ 1 ];

	if ( idBark )
		qGLWidget->deleteTexture( idBark ); //releasamo prejšnje
	if ( idLeaf )
		qGLWidget->deleteTexture( idLeaf );
	idBark = qGLWidget->bindTexture( bark );
	idLeaf = qGLWidget->bindTexture( leaf );
	izbranoDrevo->reloadIdTextures( true, true );
}

void EcosystemRender::makeTransparent( QImage *image ) {
	QRgb transparent = qRgba ( 255, 255, 255, 0 );
	for ( int x = 0; x < image->width(); x++ ) {
		for ( int y = 0; y < image->height(); y++ ) {
			QRgb value = image->pixel( x, y ); // 0xffedba31
			if ( qBlue( value ) > 200 && qRed( value ) > 200 && qGreen( value ) > 200 ) {
				image->setPixel( x, y, transparent );
			}
		}
	}
}

void EcosystemRender::toggleSingleRender() {
	singleRender = !singleRender;
	izbranoDrevo->pogled.animateGrow = false;
	izbranoDrevo->pogled.orbit = false;
	izbranoDrevo->pogled.wireframe = false;
	izbranoDrevo->pogled.wireframe = false;

	///emita ga druga kontrola:
	//emit modelDataChanged();
}

void EcosystemRender::toggleLightingEnabled() {
	lightingEnabled = !lightingEnabled;
	///emita ga druga kontrola:
	//emit modelDataChanged();
}

void EcosystemRender::toggleShowFPS() {
	showFPS = !showFPS;
}

void EcosystemRender::toggleDisplaySky() {
	displaySky = !displaySky;
}

void EcosystemRender::toggleShowNeighborHood() {
	showNeighborHood = !showNeighborHood;
}

void EcosystemRender::toggleDoNotDrawTrees() {
	doNotDrawTrees = !doNotDrawTrees;
}

void EcosystemRender::toggleWireframe() {
	wireframe = !wireframe;
	terrain->toggleWireframe();
}

void EcosystemRender::toggleDoNotRefreshDisplay() {
	doNotRefreshDisplay = !doNotRefreshDisplay;
}

void EcosystemRender::toggleDisplayTerrainNormals() {
	displayTerrainNormals = !displayTerrainNormals;
}

void EcosystemRender::toggleDisplayTerrainInclination() {
	displayTerrainInclination = !displayTerrainInclination;
}

void EcosystemRender::toggleDisplayTerrainMoisture() {
	displayTerrainMoisture = !displayTerrainMoisture;
}

void EcosystemRender::toggleDisplayTerrainSunniness() {
	displayTerrainSunniness = !displayTerrainSunniness;
}

void EcosystemRender::toggleDisplayTerrainWindiness() {
	displayTerrainWindiness = !displayTerrainWindiness;
}

void EcosystemRender::toggleDisplayLandMarkings() {
	displayLandMarkings = !displayLandMarkings;
}

void EcosystemRender::toggleDisplaySun() {
	displaySun = !displaySun;
}

bool EcosystemRender::isSingleRender() {
	return singleRender;
}

void EcosystemRender::loadSimulation( const std::string &fileName ) {
	ecosystemSimulation->loadSimulation( fileName );
	emit nrPlantsLivingChanged();
}

void EcosystemRender::saveSimulation( const std::string &fileName ) {
	ecosystemSimulation->saveSimulation( fileName );
}

void EcosystemRender::setAutosave( int nrSteps ) {
	autoSaveSimulationStepNr = nrSteps;
}

void EcosystemRender::updateAutoSave() {
	if ( autoSaveSimulationStepNr < 0 )
		return ;
	if ( ecosystemSimulation->simulationStepNr % autoSaveSimulationStepNr == 0 ) {
		std::stringstream ss;
		ss << ecosystemSimulation->simulationStepNr << ".sim";
		std::string str;
		ss >> str;
		saveSimulation( str );
	}
}

std::vector<unsigned long long> EcosystemRender::getRandomGeneratorSeeds() {
	return ecosystemSimulation->getRandomGeneratorSeeds();
}

void EcosystemRender::setRandomGeneratorSeeds( const std::vector<unsigned long long>& seeds ) {
	ecosystemSimulation->setRandomGeneratorSeeds( seeds );
}

void EcosystemRender::checkGlError( char* message ) {
	if (!checkingForGlErrors) return;
	GLenum error = 0;
#define printit(x) case x: std::cout << #x <<std::endl;break

	do {
		error = glGetError();
		if ( error == GL_NO_ERROR )
			break;
		printf( "%s: ", message );
		switch ( error ) {
			printit( GL_INVALID_ENUM );
			printit( GL_INVALID_VALUE );
			printit( GL_INVALID_OPERATION );
			printit( GL_STACK_OVERFLOW );
			printit( GL_OUT_OF_MEMORY );
			//printit( GL_TABLE_TOO_LARGE );
		}
	} while ( error != GL_NO_ERROR );
#undef printit
}

void EcosystemRender::render() {
	if ( !izbranoDrevo->pogled.animateGrow && !izbranoDrevo->pogled.animateOrbitAround &&
	        !izbranoDrevo->pogled.animateWindSway )
		emit stopTreeAnimation();

	if ( doNotRefreshDisplay )
		return ;

	static QTime timer;

	int diff = timer.restart();
	static int comulative = 0;
	static int frames = 0;
	comulative += diff;
	frames++;
	if ( comulative > 1000 ) {
		float d = 1000.0f / comulative * frames;
		if ( showFPS ) {
			std::cout << d << "fps" << std::endl;
		}
		frames = 0;
		comulative = 0;
	}

	if ( lightingEnabled )
		glEnable( GL_LIGHTING );
	else
		glDisable( GL_LIGHTING );

	checkGlError( "Zacetek renderiranja" );
	if ( singleRender ) {
		izbranoDrevo->drawSingleTreeCentered( camera->dol_pomik, camera->levo_pomik, camera->z_pomik, camera->x_zasuk, camera->y_zasuk );
	} else {
		glClearColor( 1, 1, 1.0, 1.0 ); //
		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
		glMatrixMode( GL_MODELVIEW );
		glLoadIdentity();
		checkGlError( "Identity model" );

		camera->setupEcosystemCamera();
		checkGlError( "Camera set" );
		setLightPosition();
		checkGlError( "Lights set" );
		if ( displaySky )
			sky->render();
		checkGlError( "Sky konec" );

		terrain->render();

		if ( !terrainData->isDataReady() )
			terrainData->buildTerrainData();

		if ( displayTerrainNormals )
			terrainMarkings->renderNormals();
		if ( displayTerrainInclination )
			terrainMarkings->renderInclination();
		if ( displayTerrainMoisture )
			terrainMarkings->renderMoisture();
		if ( displayTerrainSunniness )
			terrainMarkings->renderSunniness();
		if ( displayTerrainWindiness )
			terrainMarkings->renderWindiness();
		if ( displayLandMarkings )
			terrainMarkings->renderLandMarkings();
		if ( displaySun ) {
			terrainMarkings->renderSun();
		}

		renderVisiblePlants();
	}
	checkGlError( "Vse zrenderirano" );
}

int EcosystemRender::renderVisiblePlants() {
	int stRenderiranihDreves = 0;
	/// postavimo prostor za culling
	CObjectCulling objectCulling;
	CLevelOfDetail lod;

	glPushMatrix();
	{
		drevo::Matrika modelViewMatrix, projectionMatrix, nova;
		glGetFloatv( GL_MODELVIEW_MATRIX, modelViewMatrix.el );
		glGetFloatv( GL_PROJECTION_MATRIX, projectionMatrix.el );
		glLoadMatrixf( projectionMatrix.el );
		glMultMatrixf( modelViewMatrix.el );
		glGetFloatv( GL_MODELVIEW_MATRIX, nova.el );
		objectCulling.setMatrix( nova );
	}
	glPopMatrix();

	/// sporočimo pozicijo kamere za določanje LOD
	drevo::Tocka camPos;
	camPos.x = camera->x;
	camPos.y = camera->y;
	camPos.z = camera->z;

	lod.setCameraPosition( camPos );

	/// sprehodimo se po drevesih in jih upodobimo

	std::vector<int> stevilaUpodobljenihRastlin;
	std::vector<simulationEntities::CVrstaRastlin> &rastlinskeVrste = ecosystemSimulation->rastlinskeVrste;
	float maxRatio = 0;
	for ( std::vector<simulationEntities::CVrstaRastlin>::iterator vr = rastlinskeVrste.begin(); vr != rastlinskeVrste.end(); vr++ ) {
		simulationEntities::CVrstaRastlin &rastlinskaVrsta = *vr;
		if ( ( int ) rastlinskaVrsta.ziviPrimerkiTeVrste.size() == 0 )
			continue;

		int stRenderiranihDrevesVrste = 0;
		for ( simulationEntities::CVrstaRastlin::ziviPrimerkiTeVrsteIter r = rastlinskaVrsta.ziviPrimerkiTeVrste.begin();
		        r != rastlinskaVrsta.ziviPrimerkiTeVrste.end(); r++ ) {
			simulationEntities::CRastlina &rastlina = *r;
			drevo::Tocka center = rastlina.pozicija3D;
			center.y += 2.5f;

			if ( !objectCulling.containsSphere( center, 5 ) )
				continue;
			stRenderiranihDreves++;
			stRenderiranihDrevesVrste++;
			glPushMatrix();
			{
				float ratio = ( float ) rastlina.trenutnaVelikost;
				int sz = ( int ) izglediDrevesnihVrst.size();
				/// fallback: če tega drevesa nimamo, uporabimo zadnje, ki ga imamo
				int drevoGeometricModelId = rastlina.vrstaRastline->tip < sz ?
				                            rastlina.vrstaRastline->tip : sz - 1 ;
				drevo::Drevo &drevoRisano =
				    *izglediDrevesnihVrst[ drevoGeometricModelId ];
				///#define UPORABI_GL_LISTE
#ifdef UPORABI_GL_LISTE

				static int prvic[ 6 ] = {true, true, true, true, true, true};
				static GLuint glListDrevo[ 6 ];
				if ( prvic[ drevoGeometricModelId ] ) {
					glListDrevo[ drevoGeometricModelId ] = glGenLists( 1 );
					glNewList( glListDrevo[ drevoGeometricModelId ], GL_COMPILE );
					drevoRisano.drawSingleTreeAwayLOD( 0 );
					glEndList();
					prvic[ drevoGeometricModelId ] = false;
				}
#endif
				drevo::Tocka pozicijaRastline = rastlina.pozicija3D;
				glTranslatef( pozicijaRastline.x, pozicijaRastline.y, pozicijaRastline.z );
#ifdef UPORABI_GL_LISTE

				glScalef( ratio, ratio, ratio );
				glCallList( glListDrevo[ drevoGeometricModelId ] );
				continue;
#endif

				if ( showNeighborHood ) {
					glDisable( GL_DEPTH_TEST );
					glPushMatrix();
					{
						float v = rastlina.trenutniRadij;
						glRotatef( 90, 1.0, 0.0, 0.0 ); /// z postane y->disk na y=0
						glDisable( GL_TEXTURE_2D );
						glDisable( GL_LIGHTING );
						int t = rastlina.vrstaRastline->tip;
						GLubyte r = 0, g = 0, b = 0;
						switch ( t ) {
						case 0:
							r = 255;
							break;
						case 1:
							g = 255;
							break;
						case 2:
							b = 255;
							break;
						case 3:
							r = 255;
							b = 255;
							break;
						case 4:
							r = 255;
							g = 255;
							break;
						default:
							r = 255;
							g = 255;
							b = 255;
							break;
						}

						glColor3ub( r, g, b );
						gluDisk( drevoRisano.pogled.kvadraticnePloskve, 0, v, 16, 1 );
						if ( lightingEnabled )
							glEnable( GL_LIGHTING );
						else
							glDisable( GL_LIGHTING );
						glEnable( GL_TEXTURE_2D );
					}
					glPopMatrix();
					glEnable( GL_DEPTH_TEST );
				}

				if ( doNotDrawTrees )
					continue;

				drevoRisano.pogled.wireframe = wireframe;
				drevoRisano.pogled.animateWindSway = izbranoDrevo->pogled.animateWindSway;
				drevoRisano.pogled.animateGrow = true; ///vedno true: izbranoDrevo->pogled.animateGrow
				drevoRisano.pogled.animateOrbitAround = izbranoDrevo->pogled.animateOrbitAround;
				/// personalizacija glede na id rastline
				float personalRotation = rastlina.id * 17.19f;
				drevoRisano.pogled.orbit = personalRotation;

				drevoRisano.pogled.starostDrevesa = ratio > 1 ? 1 : ratio;
#ifndef max
#define max( a, b )   ((a) > (b) ? (a) : (b))
#endif

				maxRatio = max( maxRatio, ratio );
#undef max

				int lodR = lod.getLODcase( pozicijaRastline );
				drevoRisano.drawSingleTreeAwayLOD( lodR );
			}
			glPopMatrix();
		} /// for rastline
		stevilaUpodobljenihRastlin.push_back( stRenderiranihDrevesVrste );
	} /// for vrste

	ecosystemSimulation->getEntitiesStatistics() ->addRenderedSpecies( stevilaUpodobljenihRastlin );
	emit nrPlantsRenderedChanged();

	return stRenderiranihDreves;
}
