/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef ECOSYSTEMECOSYSTEMRENDER_H
#define ECOSYSTEMECOSYSTEMRENDER_H

#include <vector>
#include <QObject>
#include <vector>

class QGLContext;
class QImage;
class QGLWidget;

namespace drevo {
	class Drevo;
}

namespace ecosystem {
	class CEcosystemSimulation;
	namespace data {
		class CTerrainData;
	}
	namespace statistics {
		class CEntitiesStatistics;
	}

	/*!
	 * \namespace nature::render
	 * \brief upodabljanje s protokolom OpenGL
	 *
	 * Klici v stroj za geometrično upodabljanje so zbrani v tem imenskem prostoru.
	 * V primeru opustitve, sprememb ali dopolnitev OpenGL kninjice se spreminja le ta imenski prostor.
	 * Razredi za upodabljanje in poenostavljanje geometrije so zbrani v tem prostoru.
	 * V okolju izrie teren, vodo in obzorje. Teren se izrie s krpami, voda kot
	 * en poligon, obzorje pa kot katla. Razred Camera skrbi za določitev pogleda.
	 */
	namespace render {
		class Terrain;
		class Camera;
		class Sky;
		class TerrainMarkings;

		/**
		 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
		*/
		class EcosystemRender : public QObject {
			Q_OBJECT
		public:
			/**
			 * Kreira novi objekt v podanem OpenGL kontekstu, potrebuje
			 * tudi pripadajočo QGLWidget komponento za kreiranje tekstur.
			 * Postavi privzete parametre.
			 *
			 * Keira dve teksture za teren. Kreira ecosystemSimulation, terrainData, terrain, camera in
			 * naloži parametre dreves, ki bodo zajete v simulaciji, s teksturami delčkov dreves vred.
			 *
			 * Požene simulacije za podatke terena že takoj na začetku, ob zagonu aplikacije.
			 *
			 * @param qglContext kontekst OpenGL, v katerega riše
			 * @param qGLWidget pripadajoča komponenta za OpenGL
			 */
			EcosystemRender(const QGLContext *qglContext, QGLWidget *qGLWidget);

			/**
			 * Pobriše objekte terrain, terrainData, osvobodi ID-je teksture in pobriše naložene
			 * parametre dreves.
			 *
			 * @return
			 */
			~EcosystemRender();

			/**
			 * V postavljen OpenGL upodobi celoten ekosistem ali zgolj eno drevo, odvisno od nastavitev.
			 */
			void render();

			/**
			 * Vrne trenutno izbrano drevo, ki mu lahko urejamo parametre proceduralnega modela.
			 * @return
			 */
			drevo::Drevo* getIzbranoDrevo();
			/**
			 * Vrne trenutno kamero v EcosystemRender.
			 * @return
			 */
			Camera *getCurrentCamera();
			/**
			 * Vrne kretnico, ali trenutno renderiramo eno samo drevo ali pa renderiramo ekosistem.
			 * @return
			 */
			bool isSingleRender();
			/**
			 * Vrne shrambo za hranjeje podatkov o terenu
			 * @return
			 */
			const data::CTerrainData * getTerrainData() const;

			/**
			 * Vrne kazalec na objekt z zbrano statistiko
			 * @return zbrana statistika
			 */
			statistics::CEntitiesStatistics* getEntitiesStatistics();
			/**
			 * Naloži DMR mapo iz podane datoteke.
			 * @param fileName ime datoteke z formatom DMR (digiralni model reliefa)
			 */
			void reloadDMR(const std::string &fileName);

			/**
			 * Vrne, ali je simulacija pripravljena.
			 * @return inicializiranost simulacije
			 */
			int isSimulationRedyAndRunning();

		signals:
			/**
			 * Sporoči, da so se podatki modela ekosistema ali posameznega drevesa spremenili,
			 * kar najverjetneje povzroči osvežitev izgleda (repaint).
			 */
			void modelDataChanged();
			/**
			 * Naj začne časovnik za animacijo drevesa.
			 */
			void startTreeAnimation();
			/**
			 * Naj ustavi časovnik za animacijo drevesa.
			 */
			void stopTreeAnimation();
			/**
			 * Spremenilo se je število živih rastlin v ekosistemu
			 */
			void nrPlantsLivingChanged();
			/**
			 * Spremenilo se je število upodobljenih rastlin v ekosistemu
			 */
			void nrPlantsRenderedChanged();

			/// klicane ne-akcije sloti
		public slots:
			/**
			 * Nastavi vrednost koordinate lupe, kje naj rastejo rastline (plant area).
			 * @param val nova vrednost
			 */
			void setLupaMinX(int val);
			/**
			 * Nastavi vrednost koordinate lupe, kje naj rastejo rastline (plant area).
			 * @param val nova vrednost
			 */
			void setLupaMaxX(int val);
			/**
			 * Nastavi vrednost koordinate lupe, kje naj rastejo rastline (plant area).
			 * @param val nova vrednost
			 */
			void setLupaMinY(int val);
			/**
			 * Nastavi vrednost koordinate lupe, kje naj rastejo rastline (plant area).
			 * @param val nova vrednost
			 */
			void setLupaMaxY(int val);
			/**
			 * Vrne vrednost koordinate lupe, kje naj rastejo rastline (plant area).
			 * @return vrednost koordinate
			 */
			int getLupaMinX();
			/**
			 * Vrne vrednost koordinate lupe, kje naj rastejo rastline (plant area).
			 * @return vrednost koordinate
			 */
			int getLupaMaxX();
			/**
			 * Vrne vrednost koordinate lupe, kje naj rastejo rastline (plant area).
			 * @return vrednost koordinate
			 */
			int getLupaMinY();
			/**
			 * Vrne vrednost koordinate lupe, kje naj rastejo rastline (plant area).
			 * @return vrednost koordinate
			 */
			int getLupaMaxY();

			/**
			 * Ponovno naj naloži teksture trenutno izbranega drevesa.
			 * Uporabi dobljen qGLWidget, da kreira nove teksture.
			 */
			void reloadCurrentTreeTextures();
			/**
			 * Ponovno naj naloži teksture vseh naloženih dreves.
			 * Uporabi dobljen qGLWidget, da kreira nove teksture.
			 * Inicializira podatke na GL Contextu:
			 * - teksture,
			 * - sezname in
			 * - vertex arrays.
			 */
			void reloadAllTreeTextures();
			/**
			 * Inicializira geometrijo in teksture.
			 *
			 * Naloži vse potrebne teksture za drevesa, teren in obzorje.
			 * @param qGLWidget pripadajoča komponenta za OpenGL
			 */
			void initAllTextures(QGLWidget *qGLWidget);
			/**
			 * Preklopi kretnico, ali naj upodabi eno samo drevo ali cel ekosistem.
			 */
			void toggleSingleRender();
			/**
			 * Preklopi kretnico za uporabo luči pri senčenju.
			 */
			void toggleLightingEnabled();
			/**
			 * Simulira naslednji korak simulacije, sporoči novo statistiko o rastlinah
			 * in morda povzroči osvežitev geometrije.
			 */
			void simulateNextStepAndUpdateGL();
			/**
			 * Preklopi kretnico za prikaz števila sličic na sekundo.
			 */
			void toggleShowFPS();
			/**
			 * Preklopi kretnico za prikaz žičnega modela.
			 */
			void toggleWireframe();

			/**
			 * Preklopi kretnico za prikaz ekoloških sosednosti.
			 */
			void toggleShowNeighborHood();
			/**
			 * Preklopi kretnico za prikaz dreves v ekosistemu.
			 * Čeprav se drevesa ne prikazujejo, pa se lahko prikažejo njihove
			 * ekološke sosednosti.
			 */
			void toggleDoNotDrawTrees();
			/**
			 * Preklopi kretnico za osveževanje upodobljenega pogleda.
			 * Uporabno takrat, ko želimo zgolj poganjati simulacijo, brez upodabljanja.
			 */
			void toggleDoNotRefreshDisplay();
			/**
			 * Preklopi kretnico za prikaz normal terena.
			 */
			void toggleDisplayTerrainNormals();
			/**
			 * Preklopi kretnico za prikaz naklona.
			 */
			void toggleDisplayTerrainInclination();
			/**
			 * Preklopi kretnico za prikaz vlažnosti.
			 */
			void toggleDisplayTerrainMoisture();
			/**
			 * Preklopi kretnico za prikaz osončenosti.
			 */
			void toggleDisplayTerrainSunniness();
			/**
			 * Preklopi kretnico za prikaz vetrovnosti.
			 */
			void toggleDisplayTerrainWindiness();
			/**
			 * Preklopi kretnico za prikaz oznak terena.
			 */
			void toggleDisplayLandMarkings();
			/**
			 * Preklopi prikaz lokacije sonca.
			 */
			void toggleDisplaySun();

			/// sloti za gui-akcije
		public slots:
			/**
			 * Izbere naslednje drevo kot trenutno.
			 */
			void selectNextTreeSpecies();
			/**
			 * Poveča velikost listov trenutnega drevesa za 10%.
			 */
			void actionEnlarge_leave_size();
			/**
			 * Zmanjša velikost listov trenutnega drevesa za 10%.
			 */
			void actionReduce_leave_size();

			/**
			 * Preklopi kretnico za animacijo rasti drevesa.
			 */
			void actionGrow_tree();
			/**
			 * Preklopi kretnico za sukanje drevesa.
			 */
			void actionSpin_tree();
			/**
			 * Preklopi kretnico za zibanje drevesa v vetru.
			 */
			void actionSway_tree_in_wind();

			/**
			 * Poveča število simulacijskih korakov na en renderiran okvir.
			 */
			void actionRender_less_often_skip_more_frames();
			/**
			 * Zmanjša število simulacijskih korakov na en renderiran okvir.
			 */
			void actionRender_more_often_skip_less_frames();
			/**
			 * Preveri in izpiše morebitne OpenGL napake.
			 * @param message sporočilo za izpis ob napaki
			 */
			void checkGlError(char* message);
			/**
			 * Naloži simulacijo: lokacije in lastnosti rastlin in random generatorje.
			 * @param fileName ime datoteke, iz katere beremo
			 */
			void loadSimulation(const std::string &fileName);
			/**
			 * Shrani simulacijo: lokacije in lastnosti rastlin in random generatorje.
			 * @param fileName ime datoteke, v katero shranjujemo
			 */
			void saveSimulation(const std::string &fileName);
			/**
			 * Vrne vsa uporabljana semena za random generator.
			 * @return
			 */
			std::vector<unsigned long long> getRandomGeneratorSeeds();
			/**
			 * Nastavi semena random generatorjev simulacije
			 * @param seeds nova semena
			 */
			void setRandomGeneratorSeeds(const std::vector<unsigned long long>& seeds);
			/**
			 * Spremeni kretnico za prikaz obozorja.
			 */
			void toggleDisplaySky();
			/**
			 * Določi samodejno shranjevanje simulacije na vsakih nekaj korakov
			 * @param nrSteps nekaj korakov
			 */
			void setAutosave(int nrSteps);
			/**
			 * Če avtomatsko samodejno shranjuje, določi shranjevanje.
			 */
			void updateAutoSave();

			/**
			 * Znova naloži teksture za obzorje
			 * @param filenameSky1 sever
			 * @param filenameSky2 jug
			 * @param filenameSky3 vzhod
			 * @param filenameSky4 zahod
			 * @param filenameSky5 zgoraj
			 * @param filenameSky6 spodaj
			 */
			void reloadSkyTextures(const QString &filenameSky1,
								   const QString &filenameSky2,
								   const QString &filenameSky3,
								   const QString &filenameSky4,
								   const QString &filenameSky5,
								   const QString &filenameSky6);
			/**
			 * Naloži, pretvori in pošlje kot ID-je, teksture v Sky.
			 */
			void sendSkyTexturesToSkyRender();
			/**
			 * Vrne imena datotek, iz koder so teksture za obzorje
			 * @param filenameSky1 sever
			 * @param filenameSky2 jug
			 * @param filenameSky3 vzhod
			 * @param filenameSky4 zahod
			 * @param filenameSky5 zgoraj
			 * @param filenameSky6 spodaj
			 */
			void getSkyTextureFileNames(QString &filenameSky1,
								   QString &filenameSky2,
								   QString &filenameSky3,
								   QString &filenameSky4,
								   QString &filenameSky5,
								   QString &filenameSky6);
		private:
			/**
			 * Inicializira objekte za zaganjanje simulacije in modeliranje.
			 */
			void initModelObjects();
			/**
			 * Postavi lokacijo OpenGL luči.
			 */
			void setLightPosition();
			/**
			 * Upodobi vidne rastline. Pri tem uporablja obrezovanje na vidni
			 * volumen in geometrije za drevesa izven pogleda ne gradi.
			 * Zaenkrat ne upošteva algoritma obzorja, ki bi skril tista
			 * drevesa, ki so za hribom.
			 * @return število upodobljenih dreves
			 */
			int renderVisiblePlants();

			/**
			 * Postavi kamero, preden bo renderiral sceno.
			 */
			void setCamera();
			/**
			 * Bele piksle na sliki spremeni v prozorne, tako da priredi
			 * na alfa kanal z vrednostjo 0.
			 * @param image slika, ki naj jo spremenim.
			 */
			void makeTransparent(QImage *image);

			/**
			 * Kretnica, ali naj upodabljam le eno drevo ali celoten ekosistem.
			 */
			bool singleRender;

			/**
			 * Kretnica, ali naj upodabljam normale terena.
			 */
			bool displayTerrainNormals;
			/**
			 * Kretnica, ali naj upodabljam naklon terena.
			 */
			bool displayTerrainInclination;
			/**
			 * Kretnica, ali naj upodabljam vlažnost terena.
			 */
			bool displayTerrainMoisture;
			/**
			 * Kretnica, ali naj upodabljam osončenost terena.
			 */
			bool displayTerrainSunniness;
			/**
			 * Kretnica, ali naj upodabljam vetrovnost terena.
			 */
			bool displayTerrainWindiness;
			/**
			 * Kretnica, ali naj upodabljam označbe terena.
			 */
			bool displayLandMarkings;

			/**
			 * Objekt za postavljanje pogleda (kamere) pri
			 * OpenGL upodabljanju.
			 */
			Camera *camera;
			/**
			 * Objekt za upodabljanje terena.
			 */
			Terrain *terrain;
			/**
			 * Objekt za risanje raznih oznak terena.
			 */
			TerrainMarkings *terrainMarkings;
			/**
			 * Objekt za izris neba
			 */
			Sky *sky;
			/**
			 * Objekt za hranjenje podatkov po terenu: točke, naklon, vlaga, osončenost, vetrovnost.
			 */
			data::CTerrainData *terrainData;
			/**
			 * Simulacija ekosistema, abstraktni nivo modela z entitetami, ločen od izgleda.
			 */
			CEcosystemSimulation *ecosystemSimulation;
			/**
			 * Shranjeni parametri proceduralnih modelov dreves, ki se upodabljajo za rastlinske vrste.
			 */
			std::vector<drevo::Drevo*> izglediDrevesnihVrst;
			/**
			 * Kazalec na trenutno izbrane parametre proceduralnega modela drevesa.
			 */
			drevo::Drevo* izbranoDrevo;
			/**
			 * Indeks izbranega drevesa iz seznama modelov dreves.
			 */
			int indexIzbranegaDrevesa;

			/**
			 * Povezava na kontrolo za izris OpenGL geometrije.
			 * Uporabljena za kreiranje OpenGL tekstur.
			 */
			QGLWidget *qGLWidget;

			/**
			 * Število korakov simulacije, ki naj jih opravim, preden opravim izris
			 * enega okvirja OpenGL. Če upodobimo vsak simuliran korak, je ta vrednost 1.
			 */
			int simulationStepsPerRenderedFrame;

			/**
			 * Kretnica za postavitev luči v OpenGL.
			 */
			bool lightingEnabled;
			/**
			 * Kretnica za renderiranje le žičnega modela.
			 */
			bool wireframe;

			/**
			 * Kretnica, ali naj prikaže število sličic na sekundo.
			 */
			bool showFPS;
			/**
			 * Kretnica, ali naj prikaže kroge za prikaz ekoliških sosednosti.
			 */
			bool showNeighborHood;
			/**
			 * Kretnica, ali naj ne rišem dreves. Še vedno se lahko izrišejo krogi sosednosti.
			 */
			bool doNotDrawTrees;
			/**
			 * Kretnica, ali naj se sploh osveži OpenGL izris.
			 */
			bool doNotRefreshDisplay;
			/**
			 * Zaporedna številka trenutno simuliranega koraka.
			 * Ko število doseže 0, upodobimo prikaz.
			 */
			int simulationStepsUntilRender;
			/**
			 * Kretnica za prikaz krogle sonca (OpenGL);
			 */
			bool displaySun;
			/**
			 * Kretnica, ali naj narišemo obzorje.
			 */
			bool displaySky;
			/**
			 * Število korakov, na katere shranimo simulacijo
			 */
			int autoSaveSimulationStepNr;
			/**
			 * Ime datoteke za teksturo detajla.
			 */
			QString filenameDetail;
			/**
			 * Ime datoteke za teksturo obzorja na severu
			 */
			QString filenameSky1;
			/**
			 * Ime datoteke za teksturo obzorja na jugu
			 */
			QString filenameSky2;
			/**
			 * Ime datoteke za teksturo obzorja na vzhodu
			 */
			QString filenameSky3;
			/**
			 * Ime datoteke za teksturo obzorja na zahodu
			 */
			QString filenameSky4;
			/**
			 * Ime datoteke za teksturo obzorja zgoraj
			 */
			QString filenameSky5;
			/**
			 * Ime datoteke za teksturo obzorja spodaj
			 */
			QString filenameSky6;
			/**
			 * Ali preverja OpenGL getError
			 */
			bool checkingForGlErrors;
		};
	}
}
#endif
