/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "terrain.h"
#include "tocka.h"
#include "terraindata.h"

#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

/// tu in samo tu (zaenkrat) uporabimo glee in ne QtOpenGL, saj
/// glee omogoča še druge razširitvene funkcije, do OGL2.0
#include "GLee.h"

namespace ecosystem {
	namespace render {
		Terrain::Terrain( const QGLContext *qglContext, data::CTerrainData *terrainData ) :
		qglContext( qglContext ), terrainData( terrainData ) {
			wireframe = false;
			useDetailTexture = true;
			idTexturaTeren = 0;
			idTexturaDetail = 0;
		}

		Terrain::~Terrain() {}

		void Terrain::setTextureNames( unsigned int idTexturaTeren,
		                               unsigned int idTexturaDetail ) {
			this->idTexturaTeren = idTexturaTeren;
			this->idTexturaDetail = idTexturaDetail;
		}

		void Terrain::getTextureNames( unsigned int &idTexturaTeren,
		                               unsigned int &idTexturaDetail ) {
			idTexturaTeren = this->idTexturaTeren;
			idTexturaDetail = this->idTexturaDetail;
		}

		void Terrain::toggleWireframe() {
			wireframe = !wireframe;
		}

		void Terrain::initGLContexMultitextureAndArrays() {
			initMultitexture();
			initCompiledArray();

			/// zaklene celotni array --> ga pošlje na server
			if ( GL_EXT_compiled_vertex_array )
				glLockArraysEXT( 0, 2 * terrainData->stTockX * ( terrainData->stTockY - 1 ) );
		}
	}
	namespace render {
		void Terrain::initMultitexture() {
			float scaleDetail =    /*550.0*/ 100.0f;
			/// naložimo teksture
			stTextureArray = new float[ 2 * terrainData->stTockX * terrainData->stTockY ];
			float*p_stTextureArray = stTextureArray;
			uvTextureArray = new float[ 2 * terrainData->stTockX * terrainData->stTockY ];
			float*p_uvTextureArray = uvTextureArray;
			for ( int y = 0; y < terrainData->stTockY; y++ ) {
				for ( int x = 0; x < terrainData->stTockX; x++ ) {
					float s = float( x ) / float( terrainData->stTockX );
					float t = float( y ) / float( terrainData->stTockY );
					float u = scaleDetail * s;
					float v = scaleDetail * t;
					*p_stTextureArray++ = s;
					*p_stTextureArray++ = t;
					*p_uvTextureArray++ = u;
					*p_uvTextureArray++ = v;
				}
			}

			if ( !GLEE_ARB_multitexture )
				return ;
			/// postavimo (default) ARB multiteksturo 0 kot trenutno:
			glClientActiveTextureARB( GL_TEXTURE0_ARB );
			glTexCoordPointer( 2,     /// št. teksturnih koordinat na oglišče
			                   GL_FLOAT,      /// podatkovni tip za s,t/u,v koordinate
			                   0,      /// št. bajtov razmaka med posameznimi zapisi oglišč
			                   stTextureArray );
			glEnableClientState( GL_TEXTURE_COORD_ARRAY );

			/// postavimo ARB multiteksturo 1 kot trenutno:
			glClientActiveTextureARB( GL_TEXTURE1_ARB );
			glTexCoordPointer( 2,     /// št. teksturnih koordinat na oglišče
			                   GL_FLOAT,      /// podatkovni tip za s,t/u,v koordinate
			                   0,      /// št. bajtov razmaka med posameznimi zapisi oglišč
			                   uvTextureArray );
			glEnableClientState( GL_TEXTURE_COORD_ARRAY );

			/// vrnemo (default) ARB multiteksturo 0 kot trenutno:
			glClientActiveTextureARB( GL_TEXTURE0_ARB );

			glActiveTextureARB( GL_TEXTURE0_ARB );
		}

		void Terrain::initCompiledArray() {
			makeVertexArray();
			makeVertexIndexArray();
		}

		void Terrain::makeVertexArray() {
			/// tipi: GL_COLOR_ARRAY, GL_EDGE_FLAG_ARRAY, GL_INDEX_ARRAY, GL_NORMAL_ARRAY, GL_TEXTURE_COORD_ARRAY, and GL_VERTEX_ARRAY
			if ( !GL_EXT_vertex_array )
				return ;
			glEnableClientState( GL_VERTEX_ARRAY );
			glVertexPointer( 3,     /// št. koordinat na oglišče
			                 GL_FLOAT,      /// podatkovni tip za x,y,z koordinate
			                 0,      /// št. bajtov razmaka med posameznimi zapisi oglišč
			                 terrainData->ogliscaTerena );
		}

		void Terrain::makeVertexIndexArray() {
			/// vsaka točka doda 1 strip, razen prvih dveh.
			/// za eno vrstico krp je potrebno 2x toliko točk: 1 spodnja, 1 zgornja
			int nrIndexes = 2 * terrainData->stTockX * ( terrainData->stTockY - 1 );

			/// vertex index array je en sam client-side seznam kazalcev na točke,
			/// ki ga kasneje zaklenemo po delih
			verticesIndexArray = new unsigned short[ nrIndexes ];
			int index = 0;
			/// zgradimo naslove (kazalce na) oglišča za gradnjo stripov trikotnikov
			for ( int y = 0; y < terrainData->stTockY - 1; y++ ) {
				for ( int x = 0; x < terrainData->stTockX; x++ ) {
					/// za vsak strip 2 točki, ena iz zgornje vrstice, druga iz spodnje vrstice
					verticesIndexArray[ index++ ] = y * terrainData->stTockX + x;
					verticesIndexArray[ index++ ] = ( y + 1 ) * terrainData->stTockX + x;
				}
			}
		}

		void Terrain::bindTextures() {
			glColor3f( 1, 1, 1 );
			/// prva tekstura
			if ( GLEE_ARB_multitexture )
				glActiveTextureARB( GL_TEXTURE0_ARB );
			glEnable( GL_TEXTURE_2D );
			glBindTexture( GL_TEXTURE_2D, idTexturaTeren );
			glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL );

			/// repeat:
			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
			glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );

			/// druga tekstura
			bool detail = useDetailTexture && GLEE_ARB_multitexture;
			if ( detail ) {
				glActiveTextureARB( GL_TEXTURE1_ARB );
				glEnable( GL_TEXTURE_2D );
				glBindTexture( GL_TEXTURE_2D, idTexturaDetail );

				glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT );
				glTexEnvi( GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_ADD_SIGNED_EXT );

				/// repeat:
				glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
				glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
				glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
				glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );
			}
		}

		void Terrain::unbindTextures() {
			/// onemogočimo drugo teksturo
			bool detail = useDetailTexture && GLEE_ARB_multitexture;
			if ( detail ) {
				glDisable( GL_TEXTURE_2D );
				/// tip prve teksture nastavimo na enostavno
				glActiveTextureARB( GL_TEXTURE0_ARB );
			}

			glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE ); //textures with lightning
		}

		void Terrain::render() {
			bindTextures();
			/// izrišemo trikotnike
			if ( GL_EXT_vertex_array ) {
				for ( int y = 0; y < terrainData->stTockY - 1; y ++)
					glDrawElements( wireframe ? GL_LINE_STRIP : GL_TRIANGLE_STRIP,
					                terrainData->stTockX * 2, GL_UNSIGNED_SHORT,
					                verticesIndexArray + y * terrainData->stTockX * 2 );
			} else
				renderFallback();

			unbindTextures();
		}

		void Terrain::renderFallback() {
			bool multiTexture = GLEE_ARB_multitexture;
			for ( int y = 0; y < terrainData->stTockY - 1; y++ ) {
				glBegin( wireframe ? GL_LINE_STRIP : GL_TRIANGLE_STRIP );
				/// rišemo trikotniške stripe
				for ( int i = 0; i < terrainData->stTockX * 2; i++ ) {
					int arrLoc = y * terrainData->stTockX * 2 + i;
					unsigned short index = *(verticesIndexArray + arrLoc);
					drevo::Tocka &t1 = terrainData->ogliscaTerena[ index ];
					int locS=(index % terrainData->stTockX)*2;
					int locT=(index / terrainData->stTockY)*2;

					if (multiTexture) {
						glMultiTexCoord2f(GL_TEXTURE0_ARB, *(stTextureArray+locS), *(stTextureArray+locT));

						if (useDetailTexture) {
							glMultiTexCoord2f(GL_TEXTURE1_ARB, *(uvTextureArray+locS), *(uvTextureArray+locT));
						}
					} else glTexCoord2f(*(stTextureArray+locS), *(stTextureArray+locT));
					glVertex3f( t1.x, t1.y, t1.z );
				}
				glEnd();
			}
		}
	}
}
