/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef ECOSYSTEM_RENDERTERRAIN_H
#define ECOSYSTEM_RENDERTERRAIN_H

class QGLContext;
namespace drevo {
	class Tocka;
}

namespace ecosystem {
	namespace data {
		class CTerrainData;
	}
	namespace render {
		/*!
		 * \brief upodobitev terena, vode in obzorja
		 *
		 * Teren se upodobi s trikotniki (glDrawElements), ki jih izračuna OpenGL iz indeksov točk.
		 * Mi podamo točke na ekvidistančni mreži, ki imajo različne viine (OpenGL index arrays).
		 * Zaenkrat ne omogoča poenostavljanja geometrije terena.
		 *
		 * Vodo preprosto uprizori z enim kvadratom, tekstura je polprosojna. Odboj gričev v vodi
		 * dobimo s ponovnim renderiranjem zrcaljenega terena preko ravnine y=0.
		 * @author Ale Zamuda (ales.zamuda@uni-mb.si)
		 */
		class Terrain {
		public:
			/**
			 * Ustvari objekt in shrani povezavo na qglContext, da shranimo procAddress-e.
			 *
			 * @param qglContext Uporabljen za qglGetProcAddress
			 * @param terrainData uporabljen za lokacije ogljišč
			 */
			Terrain( const QGLContext *qglContext, data::CTerrainData *terrainData );
			/**
			 * Odstrani objekt.
			 */
			~Terrain();
			/**
			 * Shrani OpenGL imena tekstur
			 * @param idTexturaTeren OpenGL tekstura za teren
			 * @param idTexturaDetail OpenGL tekstura za detajl
			 */
			void setTextureNames( unsigned int idTexturaTeren,
								  unsigned int idTexturaDetail );
			/**
			 * Vrne OpenGL imeni tekstur
			 * @param idTexturaTeren OpenGL tekstura za teren
			 * @param idTexturaDetail OpenGL tekstura za detajl
			 */
			void getTextureNames( unsigned int &idTexturaTeren,
								  unsigned int &idTexturaDetail );
			/*!
			 * \brief Renderira teren. Riše z vertex array-em terena.
			 *
			 * Izrie poligone terena.
			 * Izrišemo shranjene točke v vertex arrayu. Izris poteka s trikotniki, ki so naslovnjeni
			 * z indeksim arrayom, ki vsebuje kazalce na točke v vertex arrayu.
			 * Ker se zadnji in naslednji prvi trikotnik v mreži ne stikata, trak trikotnikov
			 * narišemo posebej za vsako kolono/vrstico.
			 * To se stori dvakrat: prvič za navaden izris in drugič za odsev v vodi.
			 */
			void render();

			/**
			 * Fallback rutina za renderiranje, če niso podprti vertex arrays.
			 * Upošteva tudi, ali je omogočeno multitexture.
			 */
			void renderFallback();

			/*!
			 * Vrne točko, ki ni v vodi in je poljubno na neki krpi.
			 * @param resetSeed ali pri tem resetira random generator
			 * @return poljubna ustrezna toèka terena
			 */
			//data::CTerrainData::TerrainVertexData* getNearMiddlePoint(bool resetSeed);
		public:
			/**
			 * Prestavi način izrisa terena z wireframe.
			 */
			void toggleWireframe();
			/**
			 * Inicializira multitexture in compiled vertex arrays, podatki o točkeh morajo biti prej že na voljo.
			 */
			void initGLContexMultitextureAndArrays();

		private:
			/**
			 * Dobi naslove funkcij:
			 * glActiveTextureARB - za izbiro aktivne teksture, ki bo risana na objekt in njenih parametrov
			 * glClientActiveTextureARB - za izbiro teksture pri client vertex arrays
			 * glMultiTexCoord2fARB - za določanje koordinat teksture
			 */
			void initMultitexture();
			/**
			 * Inicializira prevedena polja ogljišč
			 */
			void initCompiledArray();
			/**
			 * Kreira array ogljišč.
			 * Naloži vertex,color, array v "client state".
			 */
			void makeVertexArray();
			/**
			 * Kreira array indeksov ogljišč.
			 * Vertex array v celoti alocira za teren, določi indekse in ga zaklene.
			 * Primer:
			 * Za 4x4 (16 točk):
			 * -točke:
			 *       0  1  2  3
			 * I   <
			 *       4  5  6  7
			 * II  <
			 *       8  9 10 11
			 * III <
			 *       12 13 14 15
			 * Dobimo 3 vrstice indeksov (24 indeksov):
			 *  vrstica I:   0,4; 1,5; 2,6; 3,7
			 *  vrstica II:  4,8; 5,9; 6,10; 7,11
			 *  vrstica III: 8,12; 9,13; 10,14; 11,15
			*/
			void makeVertexIndexArray();
			/**
			 * Naloži teksture za prikazan teren.
			 */
			void bindTextures();
			/**
			 * Onemogoči teksturo za detajl in naloži default teksturne parametre.
			 */
			void unbindTextures();
			/**
			 * Indeksi ogljišč.
			 */
			unsigned short* verticesIndexArray;
			/**
			 * Polje s,t koordinat teksture.
			 */
			float* stTextureArray;
			/**
			 * Polje u,v koordinat teksture.
			 */
			float* uvTextureArray;
			/**
			 * Kontekst, potreben zaradi getProcAddress.
			 */
			const QGLContext * qglContext;
			/**
			 * Kazalec na shrambo podatkov.
			 */
			data::CTerrainData *terrainData;
			/**
			 * OpenGL ID v tem OpenGL kontekstu, za teksturo izrisa grobe slike na teren.
			 */
			unsigned int idTexturaTeren;
			/**
			 * OpenGL ID v tem OpenGL kontekstu, za teksturo izrisa detajlov na teren.
			 */
			unsigned int idTexturaDetail;
			/**
			 * Kretnica za wireframe izris mreže terena.
			 */
			bool wireframe;
			/**
			 * Kretnica, ali naj uporabljamo teksturo za detajl.
			 */
			bool useDetailTexture;
		};
	}
}
#endif
