#include "rainfactory.h"
#include <algorithm>
#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

using namespace ecosystem::data;

#define WATERLEVEL_ADDON 10
#define NORMAL_WATER_RANGE 5

CTerrainData *CRainFactory::terrainDataStaticSortPtr = 0;

CRainFactory::CRainFactory( CTerrainData &terrainData ) : terrainData( terrainData ) {
	flowPower=10.0f;
}

CRainFactory::~CRainFactory( void ) {
	delete [] sortedRainNodes;
}

void CRainFactory::init() {
	terrainDataStaticSortPtr = &terrainData;
	this->nrNodes = terrainData.stTockX * terrainData.stTockY;
	sortedRainNodes = new int[ nrNodes ];
} //CRainFactory::init

void CRainFactory::buildRain() {
	init();

	makeNodesFromVertices();
	sortNodesByHeight();
	simulateWaterFlowModel();
	widenMoistureUsingInterpolation( NORMAL_WATER_RANGE );
} //CRainFactory::buildRain

void CRainFactory::makeNodesFromVertices() {
	for ( int i = 0; i < nrNodes; i++ ) {
		sortedRainNodes[ i ] = i;

		/// moisture is saved in a linear array, mapping linearly 1 : 1 with vertices
		terrainData.terrainMoisture[ i ] = 1;

		if ( terrainData.pTerrainVertexData[ i ].pos.y
		        <
			terrainData.waterHeight * terrainData.visinskaRazlikaNaTerenu ) {
			terrainData.terrainMoisture[ i ] += WATERLEVEL_ADDON;
		}
	}
} //CRainFactory::makeNodesFromVertices

bool CRainFactory::isNodeHeigher( const int &node1, const int &node2 ) {
	return
	    terrainDataStaticSortPtr->pTerrainVertexData[ node1 ].pos.y >
	    terrainDataStaticSortPtr->pTerrainVertexData[ node2 ].pos.y;
} //CRainFactory::isNodeHeigher

void CRainFactory::sortNodesByHeight() {
	std::sort( sortedRainNodes, sortedRainNodes + nrNodes, isNodeHeigher );
} //CRainFactory::sortNodesByHeight

#include <sstream>
#include <fstream>

void CRainFactory::dumpData() {
	std::ofstream izhod( "nodes.txt" );
	for ( int i = 0; i < nrNodes; i++ ) {
		std::stringstream s;
		s << i << "=" << sortedRainNodes[ i ] << "@" << "_"
		<< terrainData.pTerrainVertexData[ sortedRainNodes[ i ] ].pos.y << "_"
		<< terrainData.terrainMoisture[ sortedRainNodes[ i ] ] << "\n";
		std::string str;
		s >> str;
		izhod << str << std::endl;
	}
} //CRainFactory::dumpData

void CRainFactory::simulateWaterFlowModel() {
	for ( int i = 0; i < nrNodes; i++ ) {
		std::vector<float> neighborHeights;
		int nodeIndex = sortedRainNodes[ i ];

		float heightHere = getNeighbouringHeights( nodeIndex, neighborHeights );
		int flowNr = getNumberOfFlowAwaySides( heightHere, neighborHeights );

		/// for this node, we divide rain in rain, that stays here and rain that flows down the terrain
		divideRainForNode( nodeIndex, heightHere, neighborHeights, flowNr );
	}
} //CRainFactory::simulateWaterFlowModel

float CRainFactory::getNeighbouringHeights( int nodeIndex, std::vector<float> &neighborHeights ) {
	int resX = terrainData.stTockX;
	int x = nodeIndex % resX;
	int y = nodeIndex / resX;

	/// following directions: up, down, left, right
	neighborHeights.push_back( y > 0 ? terrainData.pTerrainVertexData[ nodeIndex - resX ].pos.y	: -1000.f );
	neighborHeights.push_back( y < resX - 1 ? terrainData.pTerrainVertexData[ nodeIndex + resX ].pos.y	: -1000.f );
	neighborHeights.push_back( x > 0 ? terrainData.pTerrainVertexData[ nodeIndex - 1 ].pos.y	: -1000.f );
	neighborHeights.push_back( x < resX - 1 ? terrainData.pTerrainVertexData[ nodeIndex + 1 ].pos.y	: -1000.f );

	return terrainData.pTerrainVertexData[ nodeIndex ].pos.y;
} //CRainFactory::getNeighbouringHeights

int CRainFactory::getNumberOfFlowAwaySides( float heightHere, std::vector<float> &neighborHeights ) {
	int flowAway = 0;

	for ( std::vector<float>::iterator i = neighborHeights.begin(); i != neighborHeights.end(); i++ ) {
		if ( *i != -1000.0f && *i < heightHere ) {
			flowAway++;
		}
	}
	return flowAway;
} //CRainFactory::getNumberOfFlowAwaySides

void CRainFactory::divideRainForNode( int nodeIndex, float heightHere, std::vector<float> &neighborHeights, int flowAwayNr ) {

	float capilarity=fabs( (terrainData.terrainInclination[ nodeIndex ]) )*flowPower;
	if (capilarity>1)
		capilarity=1;

	float flowOffHere = terrainData.terrainMoisture[ nodeIndex ] / flowAwayNr * capilarity;

	for ( std::vector<float>::iterator i = neighborHeights.begin(); i != neighborHeights.end(); i++ ) {
		if ( *i != -1000.0f && *i < heightHere ) {
			addNodeMoisture( nodeIndex, -flowOffHere );
			int neighborIndex = mapNeighborPosition( nodeIndex, ( int ) ( i - neighborHeights.begin() ) );
			addNodeMoisture( neighborIndex, flowOffHere );
		}
	}
} //CRainFactory::divideRainForNode

int CRainFactory::mapNeighborPosition( int nodeIndex, int neighborNr ) {
	int resX = terrainData.stTockX;

	int nbr = nodeIndex;
	switch ( neighborNr ) {
	case 0:
		nbr -= resX;
		break;
	case 1:
		nbr += resX;
		break;
	case 2:
		nbr--;
		break;
	case 3:
		nbr++;
		break;
	}
	return nbr;
}

void CRainFactory::addNodeMoisture( int pointIndex, float moisture ) {
	terrainData.terrainMoisture[ pointIndex ] += moisture;
} //CRainFactory::addNodeMoisture

void CRainFactory::widenMoistureUsingInterpolation( int r ) {
	for ( int k = 0; k < r; k++ ) {
		for ( int y = 1; y < ( int ) terrainData.stTockY - 1; y++ ) {
			for ( int x = 1; x < ( int ) terrainData.stTockX - 1; x++ ) {
				int pointIndex = x + y * terrainData.stTockX;

				terrainData.terrainMoisture[ pointIndex ] =
				    terrainData.terrainMoisture[ pointIndex ] +
				    terrainData.terrainMoisture[ pointIndex - 1 ] +
				    terrainData.terrainMoisture[ pointIndex + 1 ] +
				    terrainData.terrainMoisture[ pointIndex - terrainData.stTockX ] +
				    terrainData.terrainMoisture[ pointIndex + terrainData.stTockX ];

				terrainData.terrainMoisture[ pointIndex ] /= 5;
			}
		}
	}
	for ( int y = 0; y < ( int ) terrainData.stTockY; y++ )
		terrainData.terrainMoisture[ y * terrainData.stTockX ] = 0;

	for ( int x = 0; x < ( int ) terrainData.stTockX; x++ )
		terrainData.terrainMoisture[ x ] = 0;

	for ( int y = 1; y < ( int ) terrainData.stTockY - 1; y++ ) {
		for ( int x = 1; x < ( int ) terrainData.stTockX - 1; x++ ) {
			int pointIndex = x + y * terrainData.stTockX;

			if ( terrainData.terrainMoisture[ pointIndex ] > 1 )
				terrainData.terrainMoisture[ pointIndex ]=1;
		}
	}
} //CRainFactory::widenMoistureUsingInterpolation
