#ifndef ECOSYSTEM_DATA_RAIN_FACTORY_H
#define ECOSYSTEM_DATA_RAIN_FACTORY_H

#include <vector>
#include "terraindata.h"

namespace ecosystem {
	namespace data {
		/*!
		 * \brief izračun vlažnosti po modelu padavin in kapilarnosti
		 *
		 * Vlažnost dobimo iz simulacije padavin in simulacije kapilarnosti.
		 * Padavine simuliramo s stekanjem padavin po pobočjih, kapilarnost pa
		 * s širjenjem vlage od rek v okolico.
		 * Dež izračunamo tako, da na začetku vsem krpam priredimo enako vrednost
		 * deževnosti. Shranimo si indekse krp, ki so urejeni glede na nenaraščajočo
		 * y komponento točke posamezne krpe. Dež nato odlivamo na nižje ležeče krpe.
		 *
		 *
		 * Kapilarnost simuliramo tako, da rekam priredimo velike vrednosti
		 * padavin.
		 * Na koncu vlago povprečimo, da dobimo zvezno razlite vrednosti.
		 * ENG:
		 * the shadow vertices are keyed to their originals and sorted
		 * by their height, preserving the keys to originals
		 *
		 * @author Aleš Zamuda (ales.zamuda@uni-mb.si)
		 */
		class CRainFactory {
		public:
			/*!
			 * Ustvari nov objekt in shrani povezavo na shrambo faktorjev o terenu
			 * @param terrainData shramba podatkov o faktorjih terena
			 */
			CRainFactory(::ecosystem::data::CTerrainData &terrainData);
			/*!
			 * Pobriše kreiran seznam urejenosti krp po višini
			 */
			~CRainFactory(void);
			/*!
			 * Izvede simulacijo, da dobi vlažnost.
			 * Pomaga si s klici privatnih metod.
			 * Kreira in sortira referenčni seznam višine krp, simulira dež in interpolira vlažnost.
			 * ENG:
			 * // calculates rain cumulation from given terrain normals
			 * algorithm:
			 * 1. the algorithm sorts given vertices by their height, heighest first
			 * 1.1 then it iterates trough the sorted vertices, O(n)
			 * 1.2 and for each vertice, it calculates the rain that is cumulated there
			 * 2. rain at some vertice that stays there is calculated according
			 *    to 4 neighbouring vertices
			 * 2.1 the more the neighbouring vertices inclination is less than the
			 *     current one's, the more water slips away.
			 */
			void buildRain();

		private:
			/*!
			 * Poišče dimenzije terena in pripravi ustrezno shrambo za indekse referenčnega seznama
			 */
			void init();
			/*!
			 * Kreira referenčni seznam indeksov za višine krp
			 * ENG:
			 * we assign vertice indexes and
			 * set rain commulation to 1, as the rain falls equally in 2D to the surface
			 *
			 */
			void makeNodesFromVertices();
			/*!
			 * Sortira referenčni seznam indeksov po višini krp
			 */
			void sortNodesByHeight();
			/*!
			 * Primerja dve krpi po višini
			 * ENG:
			 * arranges the elements so that heigher vertices are the before lower vertices
			 * @param node1 prva krpa
			 * @param node2 druga krpa
			 * @return ali je prva krpa višja od druge
			 */
			static bool isNodeHeigher(const int &node1, const int &node2);
			/*!
			 * Izpiše debug informacije o sortiranosti krp in njihovi vlažnosti.
			 */
			void dumpData();

			/*!
			 * Simulira model zlivanja dežja.
			 * Po vrsti za vsako krpo odlije ustrezen del dežja.
			 * ENG:
			 * simulates rain water flow model, so that the rain flows to lower regions;
			 * we iterate trough heigh-sorted vertices and collect the water
			 */
			void simulateWaterFlowModel();
			/*!
			 * Dobi višine sosednjih krp, ki so višje od dane krpe.
			 * @param nodeIndex indeks dane krpe
			 * @param neighborHeights seznam za shranjevanje indeksov višjih sosednih krp
			 * @return višina same dane krpe
			 */
			float getNeighbouringHeights(int nodeIndex, std::vector<float> &neighborHeights);
			/*!
			 * Vrne število strani, na katere dež iz dane krpe lahko odteka
			 * To je število krp, ki so nižje od dane krpe.
			 * ENG:
			 * counts number of direction, that water flows in (maximum is 4 directions)
			 * @param heightHere višina trenutne krpe
			 * @param neighborHeights seznam sosednih krp
			 * @return število krp
			 */
			int getNumberOfFlowAwaySides(float heightHere, std::vector<float> &neighborHeights);
			/*!
			 * Razdeli dež iz dane krpe.
			 * ENG:
			 * if all neighbouring nodes are higher, no rain division occurs
			 * if all neighbouring nodes are lower, most rain flows away (only the rain,
			 * that is not absorbed by terrain)
			 *
			 * @param nodeIndex indeks dana krpe
			 * @param heightHere višina dane krpe
			 * @param neighborHeights seznam višin sosednjih krp
			 * @param flowAwayNr število krp, na katere dež odteka iz dane krpe
			 */
			void divideRainForNode(int nodeIndex, float heightHere,
			                       std::vector<float> &neighborHeights, int flowAwayNr);
			/*!
			 * Iz podane številke strani sosednje krpe (0..3) vrne indeks krpe, ki ji pripada.
			 * Za neko krpo imamo lahko največ 4 sosede v seznamu nijih krp.
			 * Krpa 0 je zgornja krpa, 1 je spodnja, 2 leva in 3 desna krpa.
			 * Vrnjen indeks se izračuna s pri-/odtevanjem 1 ali resolucije X.
			 * ENG:
			 * translate neighbor position from array to 2D position and finally map the
			 * 2D position in linear index position
			 *
			 * @param nodeIndex indeks obravnavane krpe
			 * @param neighborNr številka soseda, 0..3
			 * @return indeks soseda
			 */
			int mapNeighborPosition(int nodeIndex, int neighborNr);
			/*!
			 * Krpi na podanem indeksu priteje vlanost
			 * @param pointIndex indeks podane krpe
			 * @param moisture delček vlažnosti (0..100%), ki ga naj dodamo
			 */
			void addNodeMoisture(int pointIndex, float moisture);
			/*!
			 * Interpolira vlažnost z mehčanjem vrednosti.
			 * Vrednosti linearno mehča deset krat.
			 * ENG:
			 * using interpolation, we widen the effect of rain flow in pest,
			 * so that more area of surface is wet
			 *
			 * @param r
			 */
			void widenMoistureUsingInterpolation(int r);

			/*!
			 * Shranjena povezava na faktorje terena, kamor shranjujemo izračunano inklinacijo.
			 */
			CTerrainData &terrainData;
			/*!
			 * Število krp v referenčnem sezanamu višine, ki je enako številu točk terena
			 */
			int nrNodes;
			/**
			 * Kapilarnost, odvisnost moči odtekanja od nagnjenosti terena.
			 */
			float flowPower;
			/*!
			 * Povezava na objekt s faktorji terena, uporabljen za sortiranje
			 */
			static CTerrainData *terrainDataStaticSortPtr;
			/*!
			 * Po višini nenarahčajoč sortirani indeksi krp
			 */
			int* sortedRainNodes;
		};
	}
}

#endif // NATURE_DATA_RAIN_FACTORY_H
