#include "sunfactory.h"
#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

using namespace ecosystem::data;

CSunFactory::CSunFactory(CTerrainData &terrainData) : terrainData(terrainData) {
	nrSunSamples = 21;
	difuseSunlightPower = 0.2f;///20% osvetljenost terena zaradi senc, vendar difuzna pride skozi

	/// we generate sun at SOUTH, raising at EAST, falling at west
	/// x=E/W
	/// y=angle of view
	/// z=zenit angle
	sunStart = drevo::Tocka(1.0f, 0, 0.37f);
	sunEnd = drevo::Tocka(-1.0f, 0, 0.37f);
	sunMiddle = drevo::Tocka(0.0f,sinf(44.0/180.0f*M_PI), -cosf(44.0/180.0f*M_PI));

	sunStart.normalize();
	sunEnd.normalize();
	sunMiddle.normalize();
}

CSunFactory::~CSunFactory(void) {}

void CSunFactory::setNrSunSamples(int nrSunSamples) {
	this->nrSunSamples = nrSunSamples;
}//CSunFactory::setNrSunSamples

int CSunFactory::getNrSunSamples() const {
	return this->nrSunSamples;
}//CSunFactory::getNrSunSamples

void CSunFactory::setSunStart(const drevo::Tocka &sunStart) {
	this->sunStart = sunStart;
}//CSunFactory::setSunStart

drevo::Tocka CSunFactory::getSunStart() const {
	return this->sunStart;
}//CSunFactory::getSunStart

void CSunFactory::setSunEnd(const drevo::Tocka &sunEnd) {
	this->sunEnd = sunEnd;
}//CSunFactory::setSunEnd

drevo::Tocka CSunFactory::getSunEnd() const {
	return sunEnd;
}//CSunFactory::getSunEnd

drevo::Tocka CSunFactory::getSunMiddle() const {
	return sunMiddle;
}

void CSunFactory::buildSunniness() {
	float *terrainSun = terrainData.terrainSunniness;

	for (int y = 0; y < (int)terrainData.stTockY; y++) {
		for (int x = 0; x  < (int)terrainData.stTockX; x++) {
			int index = x + y * terrainData.stTockX;
			drevo::Tocka &normal = terrainData.terrainNormals[index];
			drevo::Tocka2D tocka(x,y);
			terrainSun[index] = getPointCumulativeSunniness(tocka, normal);
		}
	}
}//CSunFactory::buildSunniness

float CSunFactory::getPointCumulativeSunniness(const drevo::Tocka2D &startingLocation, const drevo::Tocka &normal) const {
	float sum = 0;

	for (int i = 0; i < nrSunSamples; i++) {
		drevo::Tocka sunPosition = getSunPositionAtSample(i);
		float cosine = getCosineBetweenNormalisedVectors(normal, sunPosition);
		float x = getSunPower(cosine);

		///predpostavimo, da je št. samplov liho število in vzrorce utežimo
		if (i <= (nrSunSamples-1)/2) {
			x*=1.0f+float(2*i)/float(nrSunSamples-1);
		} else {
			x*=3.0f-float(2*i)/float(nrSunSamples-1);
		}

		/// upoštevamo še shelter
		x*=calculateShelternessBehindTerrain(startingLocation, sunPosition);
		sum += x;
	}

	return sum / (float)nrSunSamples;
}//CSunFactory::getPointSunniness

drevo::Tocka CSunFactory::getSunPositionAtSample(int i) const {
	if (i==0)
		return sunStart;
	if (i==nrSunSamples-1)
		return sunEnd;
	if (i==(nrSunSamples-1)/2)
		return sunMiddle;

	drevo::Tocka sunVector(0,0,0);
	/// od vzhoda do poldneva
	if (i < (nrSunSamples-1)/2) {
		drevo::Tocka sunDiff=sunMiddle-sunStart;

		///izračunamo relativno lokacijo (x,y,z) na ravnini
		float k = float(i)/(nrSunSamples/2);
		float fi = float(M_PI_2)*k;
		float k_x = cos(fi); /// -- krožno odvisno
		float k_z = sin(fi); /// -- krožno odvisno
		float k_y = k;/// -- linearno odvisno

		/// relativno lokacijo pomnožimo, da dobimo pravo lokacijo
		sunVector = sunStart+drevo::Tocka((1-k_x)*sunDiff.x, k_y*sunDiff.y, k_z*sunDiff.z);
	}
	/// od poldneva do zahoda
	else if (i < nrSunSamples) {
		drevo::Tocka sunDiff=sunEnd-sunMiddle;

		///izračunamo relativno lokacijo (x,y,z) na ravnini
		float k=float(i-((nrSunSamples)/2))/((nrSunSamples)/2);
		float fi = float(M_PI_2)*k;
		float k_x = sin(fi); /// -- krožno odvisno
		float k_z = cos(fi); /// -- krožno odvisno
		float k_y = k;/// -- linearno odvisno

		/// relativno lokacijo pomnožimo, da dobimo pravo lokacijo
		sunVector = sunMiddle+drevo::Tocka(k_x*sunDiff.x, k_y*sunDiff.y, (1-k_z)*sunDiff.z);
	}

	sunVector.normalize();
	return sunVector;
}//CSunFactory::getSunPositionAtSample

float CSunFactory::getCosineBetweenNormalisedVectors(const drevo::Tocka &normal, const drevo::Tocka &sunPosition) const {
	return normal.skalarniProdukt(sunPosition);
}//CSunFactory::getAngleBetweenVectorsAtPosition

float CSunFactory::getSunPower(float cosine) const {
	return cosine >= 0 ? cosine : 0;
}//CSunFactory::getSunPower

/// od tu naprej je samo koda za računanje senc:

float CSunFactory::calculateShelternessBehindTerrain(const drevo::Tocka2D &startingLocation,
        const drevo::Tocka &sunVector) const {
	drevo::Tocka2D currentLocation = startingLocation;
	/// angle is 0..90 = 0..PI/2
	float sunAngle=asinf(sunVector.y/sunVector.dolzina());

	/// we move forward, calculate angle from here to starting point and collect the maximum value
	while (getNextVertex(currentLocation, sunVector)) {
		float angle = getAngle(startingLocation, currentLocation);
		if (angle > sunAngle)
			return difuseSunlightPower;
	}

	return 1;//sunComesTrough;
}//CSunFactory::calculateShelternessBehindTerrain

bool CSunFactory::getNextVertex(drevo::Tocka2D &currentLocation, const drevo::Tocka &sunVector) const {
	/// we take krpa direction and add it to index, meaning, we are going towards sun
	currentLocation.x += sunVector.x;
	currentLocation.y += sunVector.z;

	/// if vertex is still on the map, we return true, to continue the iteration operation
	/// if vertex falls off map, we stop the iteration: we retun false
	return
	    currentLocation.x >= 0 && currentLocation.x < terrainData.stTockX &&
	    currentLocation.y >= 0 && currentLocation.y < terrainData.stTockY;
}//CSunFactory::getNextVertex

float CSunFactory::getAngle(const drevo::Tocka2D &startingLocation, const drevo::Tocka2D &currentLocation) const {
	drevo::Tocka p1, p2;
	p1.x = (float)startingLocation.x*(terrainData.dimen.x/terrainData.stTockX);
	p1.z = (float)startingLocation.y*(terrainData.dimen.z/terrainData.stTockY);
	p1.y = terrainData.pTerrainVertexData[(int)(startingLocation.x + startingLocation.y * terrainData.stTockX)].pos.y;

	p2.x = (float)currentLocation.x*(terrainData.dimen.x/terrainData.stTockX);
	p2.z = (float)currentLocation.y*(terrainData.dimen.z/terrainData.stTockY);
	p2.y = terrainData.pTerrainVertexData[(int)(currentLocation.x + currentLocation.y * terrainData.stTockX)].pos.y;

	if (p2.y < p1.y)
		return -1.0f;/// point is lower

	drevo::Tocka diff = p2-p1;
	drevo::Tocka proj = diff;
	float y = proj.y;
	proj.y = 0;/// project the point to plane y=0

	float tanAlpha = y / proj.dolzina();

	return atan(tanAlpha);
}//CSunFactory::getAngle
