#ifndef ECOSYSTEM_DATA_SUN_FACTORY_H
#define ECOSYSTEM_DATA_SUN_FACTORY_H

#include <vector>
#include "terraindata.h"
#include "tocka.h"
#include "tocka2d.h"

namespace ecosystem {
	namespace data {
		/*!
		 * \brief izraču sončosti z modelom potovanja sonca
		 *
		 * Sonce potuje iz vzhoda proti zahodu, mi pa vzorčimo vplive sonca.
		 * Vplive seštejemo in dobimo vpliv sonca za neko krpo.
		 * Parametri modela so strani neba, število vzorcev za sončnost,
		 * geografska širina in odbojnost svetlobe na terenu.
		 *
		 * ENG:
		 * Using this class we determine terrain's sunniness.
		 * To put it other way, if a point is directed towards sun, it is more sunny.
		 * The model simulates sun moving from east to west and we sample the intensity.
		 *@author Aleš Zamuda (ales.zamuda@uni-mb.si)
		 */
		class CSunFactory {
		public:
			/*!
			 * Kreira nov objekt in shrani povezavo na shrambo faktorjev terena.
			 * @param terrainData shramba faktorjev terena
			 */
			CSunFactory(CTerrainData &terrainData);
			/*!
			 * Destruktor, ne naredi nič
			 */
			~CSunFactory(void);

			/*!
			 * Sproži simulacijo potovanja sonca.
			 * ENG:
			 * For each vertex, we sample few times to get the mean value of
			 * the sunniness, calculated using the angle between terrain normal
			 * and the sun's vector, which gets turned for some degrees with
			 * each sampling: this simulates the sun's moving accross horizon.
			 * The sampling means that some areas might be in dark at morning,
			 * but will maby comulate much sun at noon or in the afternoon
			 */
			void buildSunniness();

			/*!
			 * @param nrSunSamples število vzorcev sempliranja položaja sonca
			 */
			void setNrSunSamples(int nrSunSamples);
			/*!
			 * @return število vzorcev sempliranja položaja sonca
			 */
			int getNrSunSamples() const;

			/*!
			 * @param sunStart začetni (jutranji) položaj sonca
			 */
			void setSunStart(const drevo::Tocka &sunStart);
			/*!
			 * @return začetni (jutranji) položaj sonca
			 */
			drevo::Tocka getSunStart() const;
			/*!
			 * @param sunEnd končni (večerni) položaj sonca
			 */
			void setSunEnd(const drevo::Tocka &sunEnd);
			/*!
			 * @return končni (večerni) položaj sonca
			 */
			drevo::Tocka getSunEnd() const;
			/**
			 * Vrne srednjo točko sonca.
			 * @return srednja točka sonca
			 */
			drevo::Tocka getSunMiddle() const;
			/*!
			 * Določa položaj sonca
			 *
			 * ENG:
			 * // For a given position index (0..nrSunSamples) returns the sun's position,
			 * that is the vector pointing in to sun from the terrain,
			 * Remarks: sun is so far that terrain's position is irrelevant to sun's position,
			 * this is as terrain would be a size of a point.
			 *
			 * @param i številka vzorca
			 * @return vrne položaj sonca na tej številki vzorca
			 */
			drevo::Tocka getSunPositionAtSample(int i) const;

		private:
			/*!
			 * Določa vrednost faktorja sončnosti na tej krpi
			 *
			 * ENG:
			 * For a given position and its normal, it samples sun's position and
			 * returns average of method calls for the sunniness at sampled position
			 *
			 * @param startingLocation indeks krpe, kjer začnemo z iskanjem
			 * @param normal normala krpe
			 * @return sončnost krpe
			 */
			float getPointCumulativeSunniness(const drevo::Tocka2D &startingLocation, const drevo::Tocka &normal) const;
			/*!
			 * Vrne arccos (dotproduct) med podanima vektorjema
			 * ENG:
			 * arcccosine between vectors equal to dot product of these vectors
			 *
			 * @param normal normala krpe
			 * @param sunPosition vektor v smeri položaja sonca
			 * @return arccos kota med vektorjema
			 */
			float getCosineBetweenNormalisedVectors(
			    const drevo::Tocka &normal,
			    const drevo::Tocka &sunPosition) const;
			/*!
			 * Določi moč sonca iz dobljenega arccos kota.
			 *
			 * ENG:
			 * For a given position index (0..nrSunSamples) returns the sun's power.
			 * The value gets normalized to 0..1.
			 *
			 * @param cosine arccos kota do sonca
			 * @return moč sonca za dan kot na obravnavani krpi
			 */
			float getSunPower(float cosine) const;
			/*!
			 * Iz danega indeksa krpe izračuna maksimalni kosinus kota proti točki na premici proti soncu.
			 * Gremo do konca terena, dokljer imamo podatke.
			 *
			 * @param startingLocation lokacija dane krpe
			 * @return koliko je krpa v senci
			 */
			float calculateShelternessBehindTerrain(const drevo::Tocka2D &startingLocation, const drevo::Tocka &sunVector) const;
			/*!
			 * Vrne kot med danima lokacijama dveh krp, za izračun kota osončenosti.
			 * Izračun poteka med 3D točkama, ki sta določeni s 2D indeksoma.
			 * @param startingLocation začetna lokacija (indeks krpe)
			 * @param currentLocation trenutna lokacija (indeks krpe)
			 * @return
			 */
			float getAngle(const drevo::Tocka2D &startingLocation, const drevo::Tocka2D &currentLocation) const;
			/*!
			 * Vrne naslednji vzorec (indeks krpe) rastra na premici proti soncu
			 * @param currentLocation vrnjen vzorec
			 * @param sunVector vzorec sonca, proti kateremu trenutno gremo
			 * @return ali smo vzeli zadnji vzorec
			 */
			bool getNextVertex(drevo::Tocka2D &currentLocation, const drevo::Tocka &sunVector) const;
			/*!
			 * Povezava na podatke o terenu, za shranjevanje faktorja sonca
			 */
			CTerrainData  &terrainData;
			/*!
			 * število vzorcev sonca, ki naj jih vzamemo v simulaciji
			 */
			int nrSunSamples;
			/**
			 * Osvetljenost krpe iz vzorca (odstotek), če je ona v senci, vendar difuzna svetloba še pride do tja
			 * Tako osvetljenost krp še vedno ostane nekoliko odvisna od naklona (od 0%-20%)
			 * pri krpah v senci.
			 */
			float difuseSunlightPower;
			/*!
			 * Začetni (jutranji) položaj sonca, na robu obzorja na vzhodu.
			 */
			drevo::Tocka sunStart;
			/*!
			 * Končni (večerni) položaj sonca, na robu obzorja na zahodu.
			 */
			drevo::Tocka sunEnd;
			/*!
			 * Vmesni (opoldanski) položaj sonca, dvignjen nad oborje.
			 */
			drevo::Tocka sunMiddle;
		};
	}//namespace data
}//namespace ecosystem
#endif // ECOSYSTEM_DATA_SUN_FACTORY_H
