#ifndef ECOSYSTEM_DATA_TERRAIN_DATA_H
#define ECOSYSTEM_DATA_TERRAIN_DATA_H

#include "tocka.h"
#include "tocka2i.h"

#include <vector>
class QImage;

namespace ecosystem {
	/*!
	 * \namespace ecosystem::data
	 * \brief geometrija in ekološki faktorji nežive narave
	 *
	 * Hrani podatke nežive narave, kot so:
	 * - dimenzija obravnavanega sveta,
	 * - geometrija terena,
	 * - položaji krp in
	 * - normale, naklon, vlaga, sončnost, vetrovnost v krpah.
	 *
	 * Podatke nastavljamo s tovarnami (programski vzorec).
	 */
	namespace data {
		class CHeighMapFactory;
		class CTerrainNormalsFactory;
		class CInclinationFactory;
		class CRainFactory;
		class CWindFactory;
		class CSunFactory;

		/*!
		 * \brief shramba vseh faktorjev terena
		 *
		 * Hrani točke terena, indekse krp nad vodo, normale krp,
		 * naklone krp, vlažnost krp, vetrovnost krp in sončnosti krp.
		 *
		 * Vrednosti se postavijo s tovarnami tako, da jim posredujemo
		 * ta objekt.
		 *
		 *@author Aleš Zamuda (ales.zamuda@uni-mb.si)
		 */
		class CTerrainData {
		public:
			/*!
			 * Kazalce seznamov faktorjev postavi na 0.
			 */
			CTerrainData( void );
			/*!
			 * Pobriše shranjene sezname faktorjev.
			 */
			~CTerrainData( void );
			/*!
			 * Zgradi faktorje okolja terena.
			 * Izračuna, kje so krpe, ki niso v vodi, proži klice za določitev normal terena,
			 * naklona, padavin, sonca in vetrovnosti.
			 *
			 */
			void buildTerrainData();
			/**
			 * Omogoča zamenjavo trenutne DMR mape in ponovno zažene simulacije lastnosti terena.
			 * @param fileName datoteka, iz katere naj naložimo novo DMR mapo
			 */
			void reloadDMR(char* fileName);

			/**
			 * Hrani max x podatek o mejah naloženeha DMR modela za točke terena.
			 */
			float dmrMaxX;
			/**
				 * Hrani max y podatek o mejah naloženeha DMR modela za točke terena.
			 */
			float dmrMaxY;
			/**
				 * Hrani max z podatek o mejah naloženeha DMR modela za točke terena.
			 */
			float dmrMaxZ;
			/**
				 * Hrani min x podatek o mejah naloženeha DMR modela za točke terena.
			 */
			float dmrMinX;
			/**
				 * Hrani min x podatek o mejah naloženeha DMR modela za točke terena.
			 */
			float dmrMinY;
			/**
				 * Hrani min z podatek o mejah naloženeha DMR modela za točke terena.
			 */
			float dmrMinZ;
			/**
			 * Število metrov razlike med najvišjo in najnižjo točko
			 */
			float visinskaRazlikaNaTerenu;

			/*!
			 * Izhodihče terena (0,0,0)
			 */
			drevo::Tocka origin;
			/*!
			 * Dimenzije (world dimensions) terena, npr. 1000x1000x260 metrov.
			 */
			drevo::Tocka dimen;
			/*!
			 * Dimenzije terena, število krp.
			 */
			int stTockX, stTockY;
			/*!
			 * Nivo vode (0..1 glede na max), dimenzija višine terena.
			 */
			float waterHeight;

			/*!
			 * Podatki o krpi z vidika OpenGL:
			 * pozicija, barva, po dve koordinati
			 */
			struct TerrainVertexData {
				drevo::Tocka	pos;/// samo lokacija točk
			};
			/*!
			 * Objekt za podatke o točkah
			 */
			TerrainVertexData *pTerrainVertexData;

			/*!
			 * Seznam indeksov krp, ki so nad vodo
			 */
			std::vector<int> nonWaterIndexes;
			/**
			 * Podatki o ogliščih terena: višina in lokacija točk.
			 */
			drevo::Tocka *ogliscaTerena;
			/*!
			 * Vektorji normal terena, gradi jih tovarna
			 */
			drevo::Tocka *terrainNormals;
			/*!
			 * Faktorji naklona terena, gradi jih tovarna
			*/
			float *terrainInclination;
			/*!
			 * Faktorji vlažnosti terena, gradi jih tovarna
			 */
			float *terrainMoisture;
			/*!
			 * Faktorji vetrovnosti terena, gradi jih tovarna
			 */
			float *terrainWindiness;
			/*!
			 * Faktorji sončnosti terena, gradi jih tovarna
			 */
			float *terrainSunniness;
			/**
			 * Ortofoto tekstura prikazanega terena.
			 * Ob zagonu jo iz DMR višin in 4 slik zgenerira CHeighMapFactory.
			 */
			QImage *ortofotoTexture;
			/**
			 * Omejitev min(X) v številu krp, kjer lahko rastline rastejo.
			 * Upoštevana pri izračunu buildNonwaterIndexes, ki potem le
			 * tu generira indekse nad vodo.
			 */
			int lupaXmin;
			/**
			 * Omejitev max(X) v številu krp, kjer lahko rastline rastejo.
			 * Upoštevana pri izračunu buildNonwaterIndexes, ki potem le
			 * tu generira indekse nad vodo.
			 */
			int lupaXmax;
			/**
			 * Omejitev min(Y) v številu krp, kjer lahko rastline rastejo.
			 * Upoštevana pri izračunu buildNonwaterIndexes, ki potem le
			 * tu generira indekse nad vodo.
			 */
			int lupaYmin;
			/**
			 * Omejitev max(Y) v številu krp, kjer lahko rastline rastejo.
			 * Upoštevana pri izračunu buildNonwaterIndexes, ki potem le
			 * tu generira indekse nad vodo.
			 */
			int lupaYmax;

			/*!
			 * Vrne naključno pozicijo med dvema danima vektorjema i in j, s krajišim v A
			 * @param A izhodišča toča krpe
			 * @param i točka po x
			 * @param j točka po z
			 * @return naključn točka na krpi
			 */
			drevo::Tocka vrniNakljucnoPozicijo( drevo::Tocka &A, drevo::Tocka &i, drevo::Tocka &j );
			/*!
			 * Iz 2d pozicije na mapi terena vrne naključno pozicijo v 3d,
			 * ki je na kvadratu v tej 2d poziciji.
			 * @param pozicija2D dana krpa, kamor damo rastlino
			 * @param pozicija3d določena nova lokacija rastline
			 */
			void dolociPozicijoNoveRastline3D( const Tocka2i &pozicija2D, drevo::Tocka &pozicija3d );
			/*!
			 * Pove, ali je točko določena 3D pozicija nad vodo
			 * Uporablja se pri sajenju rastlin
			 * @param pozicija3D točka terena
			 * @return ali je točka višje od gladine vode
			 */
			bool jeNadVodo( const drevo::Tocka &pozicija3D );
			/*!
			 * Pove, ali je dana 2D pozicija nad vodo
			 * Uporablja se pri sajenju rastlin
			 * @param pozicija2D točka terena
			 * @return ali je točka višje od gladine vode
			 */
			bool jeNadVodo( const Tocka2i &pozicija2D );
			/*!
			 * Random za enakomerno porazdeljeno naključno
			 * določanje 3D lokacije rastline na krpi, po I
			 */
			static unsigned long long randomPozicije3DNovihRastlinI;
			/*!
			 * Random za enakomerno porazdeljeno naključno
			 * določanje 3D lokacije rastline na krpi, po J
			 */
			static unsigned long long randomPozicije3DNovihRastlinJ;
			/**
			 * Vrne, ali so podatki majhnih simulacij lastnosti
			 * terena že izračunani in shranjeni.
			 */
			bool isDataReady();
			/**
			 * Vrne sunFactory, potreben pri izrisovanju položajev sonca.
			 * @return sunFactory
			 */
			const CSunFactory* getSunFactory() const;

			/*!
			 * Zgradi seznam indeksov krp, ki so nad vodo.
			 */
			void buildNonwaterIndexes();
		private:
			/**
			 * Določi lupo za rast rastlin.
			 */
			void spawnLupa();
			/**
			 * Tovarna za izdelavo ogljišč terena.
			 */
			CHeighMapFactory *heighMapFactory;
			/*!
			* Tovarna za izdelavo faktorja normal terena
			 */
			CTerrainNormalsFactory *terrainNormalsFactory;
			/*!
			 * Tovarna za izdelavo faktorja naklona terena
			 */
			CInclinationFactory *inclinationFactory;
			/*!
			 * Tovarna za izdelavo faktorja vlažnosti terena
			 */
			CRainFactory *rainFactory;
			/*!
			 * Tovarna za izdelavo faktorja vetrovnosti terena
			 */
			CWindFactory *windFactory;
			/*!
			 * Tovarna za izdelavo faktorja sončnosti terena
			 */
			CSunFactory *sunFactory;
			/**
			 * Kretnica, ali so terenski podatki izračunani (sonce, veter,..)
			 */
			bool dataIsReady;
		};
	} //namespace data
} //namespace ecosystem
#endif // ECOSYSTEM_DATA_TERRAIN_DATA_H
