#include "windfactory.h"
#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

using namespace ecosystem::data;

CWindFactory::CWindFactory(CTerrainData &terrainData) : terrainData(terrainData) {
	/// initialization: winds blow from North
	windVector.x = 0;
	windVector.y = 1; /// from 2D map: y=z is in 3D world

	windFallRadius = 10; /// 100m
	windFallAngle = float(40*(M_PI/180));
}

CWindFactory::~CWindFactory(void) {}

const drevo::Tocka2D& CWindFactory::getWindVector() const {
	return windVector;
}//CWindFactory::getWindVector

int CWindFactory::getWindFallRadius() const {
	return windFallRadius;
}//CWindFactory::getWindFallRadius

void CWindFactory::setWindFallRadius(int windFallRadius) {
	this->windFallRadius = windFallRadius;
}//CWindFactory::setWindFallRadius

float CWindFactory::getWindFallAngle() const {
	return windFallAngle;
}//CWindFactory::getWindFallAngle

void CWindFactory::setWindFallAngle(float windFallAngle) {
	this->windFallAngle = windFallAngle;
}//CWindFactory::setWindFallAngle

void CWindFactory::buildWindiness() {
	for (int y = 0; y < (int)terrainData.stTockY; y++) {
		for (int x = 0; x  < (int)terrainData.stTockX; x++) {
			int index = x + y * terrainData.stTockX;
			drevo::Tocka2D tocka(x,y);
			terrainData.terrainWindiness[index] = calculateWindness(tocka);
		}
	}
}//CWindFactory::buildWindiness

float CWindFactory::calculateWindness(drevo::Tocka2D &startingLocation) {
	drevo::Tocka2D currentLocation = startingLocation;
	float maxAngle = 0;
	int stepsTaken = 0;

	/// we move forward, calculate angle from here to starting point and collect the maximum value
	while (getNextVertex(currentLocation)) {
		float angle = getAngle(startingLocation, currentLocation);
		if (angle > maxAngle)
			maxAngle = angle;
		if (++stepsTaken >= windFallRadius)
			break;
	}

	/// angle is 0..90 = 0..PI/2
	float shelter =
	    maxAngle > windFallAngle ?
	    1.0f : maxAngle / windFallAngle /*float(M_PI_2)*/;

	float windness = 1 - shelter;

	return windness;
}//CWindFactory::calculateWindness

bool CWindFactory::getNextVertex(drevo::Tocka2D &currentLocation) const {
	/// we take wind direction and substracting it to index, meaning, we are going towards wind
	currentLocation.x -= windVector.x;
	currentLocation.y -= windVector.y;

	/// if vertex is still on the map, we return true, to continue the iteration operation
	/// if vertex falls off map, we stop the iteration: we retun false
	return
	    currentLocation.x >= 0 && currentLocation.x < terrainData.stTockX &&
	    currentLocation.y >= 0 && currentLocation.y < terrainData.stTockY;
}//CWindFactory::getNextVertex

float CWindFactory::getAngle(const drevo::Tocka2D &startingLocation, const drevo::Tocka2D &currentLocation) const {
	drevo::Tocka p1, p2;
	p1.x = (float)startingLocation.x*(terrainData.dimen.x/terrainData.stTockX);
	p1.z = (float)startingLocation.y*(terrainData.dimen.z/terrainData.stTockY);
	p1.y = terrainData.pTerrainVertexData[(int)(startingLocation.x + startingLocation.y * terrainData.stTockX)].pos.y;

	p2.x = (float)currentLocation.x*(terrainData.dimen.x/terrainData.stTockX);
	p2.z = (float)currentLocation.y*(terrainData.dimen.z/terrainData.stTockY);
	p2.y = terrainData.pTerrainVertexData[(int)(currentLocation.x + currentLocation.y * terrainData.stTockX)].pos.y;

	if (p2.y < p1.y)
		return -1.0f;/// point is lower

	drevo::Tocka diff = p2-p1;
	drevo::Tocka proj = diff;
	float y = proj.y;
	proj.y = 0;/// project the point to plane y=0

	float tanAlpha = y / proj.dolzina();

	return atan(tanAlpha);
}//CWindFactory::getAngle
