#ifndef ECOSYSTEM_DATA_WIND_FACTORY_H
#define ECOSYSTEM_DATA_WIND_FACTORY_H

#include <vector>
#include "terraindata.h"
#include "tocka2d.h"

namespace ecosystem {
	namespace data {
		/*!
		 * \brief določitev vetrovnosti
		 *
		 * Vetrovnost se določi z upoštevanjem zavetrja, smeri vetra in spuščnja vetra.
		 *@author Aleš Zamuda (ales.zamuda@uni-mb.si)
		 */
		class CWindFactory {
		public:
			/*!
			 * Inicializira parametre in shrani povezavo na shrambo faktorjev.
			 * Inicializira smer vetra ter radij in kot 100% spuščanja vetra.
			 * @param terrainData povezava na shrambo faktorjev
			 */
			CWindFactory(::ecosystem::data::CTerrainData &terrainData);
			/*!
			 * Destruktor, ne naredi nič
			 */
			~CWindFactory(void);

			/*!
			 * Zgradi vetrovnosti, tako da za določen radij sledi premici proti vetru.
			 * Rasterizira položaje na premici in jemlje vzorce (to so indeksi krp).
			 * Iz višin krp na dobljenih indeksih išče maksimalni kot.
			 * Veter izračunamo iz maksimalnega kota.
			 *
			 * ENG:
			 * Towards wind, for a specified radius we follow a line.
			 * For each vertex in a line pointing towards wind, we extract it's height and
			 * calculate it's angle.
			 * The maximum calculated angle is used for wind calculation.
			 */
			void buildWindiness();
			/*!
			 * @return vektor v smeri snopa vektorjev vetra
			 */
			const drevo::Tocka2D& getWindVector() const;
			/*!
			 * @return radij sledenja premice proti vetru
			 */
			int getWindFallRadius() const;
			/*!
			 * @param windFallRadius radij sledenja premice proti vetru
			 */
			void setWindFallRadius(int windFallRadius);
			/*!
			 * @return kot, v katerem se zrak spusti toliko, da veter zapiha 100%
			 */
			float getWindFallAngle() const;
			/*!
			 * @param windFallAngle kot, v katerem se zrak spusti toliko, da veter zapiha 100%
			 */
			void setWindFallAngle(float windFallAngle);

		private:
			/*!
			 * Iz danega indeksa krpe izračuna maksimalni kot proti točki na premici proti vetru
			 * @param startingLocation lokacija dane krpe
			 * @return veter, ki piha na to krpo
			 */
			float calculateWindness(drevo::Tocka2D &startingLocation);
			/*!
			 * Vrne naslednji vzorec (indeks krpe) rastra na premici proti vetru
			 * @param currentLocation vrnjen vzorec
			 * @return ali smo vzeli zadnji vzorec
			 */
			bool getNextVertex(drevo::Tocka2D &currentLocation) const;
			/*!
			 * Vrne kot med danima lokacijama dveh krp, za izračun vetrovnosti.
			 * Izračun poteka med 3D točkama, ki sta določeni s 2D indeksoma.
			 * @param startingLocation začetna lokacija (indeks krpe)
			 * @param currentLocation trenutna lokacija (indeks krpe)
			 * @return
			 */
			float getAngle(const drevo::Tocka2D &startingLocation, const drevo::Tocka2D &currentLocation) const;

			/*!
			 * Vektor v smeri pihanja vetra
			 */
			drevo::Tocka2D windVector;
			/*!
			 * Radij iskanja za največjim kotom.
			 */
			int windFallRadius;
			/*!
			 * Kot, za katerim nastopi popolno zavetrje
			 */
			float windFallAngle;

			/*!
			 * Povezava na shrambo faktorjev
			 */
			CTerrainData &terrainData;
		};
	}//namespace data
}//namespace ecosystem
#endif // ECOSYSTEM_DATA_WIND_FACTORY_H
