#ifndef ECOSYSTEM_SIMULATION_H
#define ECOSYSTEM_SIMULATION_H

#include "terraindata.h"
#include "vrstarastlin.h"
#include "rastlina.h"
#include "lifefactors.h"
#include "entitiesstatistics.h"

#include <QTime>
#include <vector>
#include <string>

/*!
*
* \namespace ecosystem
* \brief simulacija ekosistema
*
* Izvaja stohastično simulacijo življenja individuumov ekosistema.
* V ekosistemu so zaenkrat edini individuumi rastline večih rastlinskih vrst, brez živali.
* Rastline po ekosistemu rastejo v odvisnosti od pogojejev terena (paket ecosystem::data)
* in medsebojne interakcije (paket ecosystem::simulationEntities).
* Paket ecosystem omogoča:
* - prikaz dreves s spreminjajočo 3D geometrijo v realnem času,
* - prikaz realističnega terena in obzorja okolja in
* - simulacijo ekosistema.
*/
namespace ecosystem {
	/*!
	* \brief nadzorna rutina simulacije
	*
	* Koračno simulira razvoj ekosistema:
	* - vse rastline umrejo, ko dosežejo točno določno starost,
	* - rojevajo se fiksno,
	* - rast je fiksna in
	* - rastlini se prekrivata, če sta znotraj soseščin (enakost tudi šteje).
	*
	*@author Aleš Zamuda (ales.zamuda@uni-mb.si)
	*/
	class CEcosystemSimulation {
	public:
		/*!
		* Inicializira korak simulacije na 0 in postavi pripravljeno na zagon simulacije
		*/
		CEcosystemSimulation();
		/*!
		* Izbriše seznam rastlin po terenu
		*/
		~CEcosystemSimulation(void);
		/*!
		* Shrani kazalec na teren in kreira seznam rastlin po terenu ter ga polje v lifeFactors
		* @param terrainData kazalec na podatke o terenu
		*/
		void setTerrainData(::ecosystem::data::CTerrainData &terrainData);
		/*!
		* Glavna metoda, za simuliranje naslednjega korakoma: KREIRAMO, RASTEMO, ODSTRANJUJEMO rastline
		*/
		void simulateNextStep();

		/**
		 * Kretnica, ali simulacija že teče, ali pa jo bo šele potrebno inicializirati.
		 */
		bool simulationRunning;
		/**
		 * Številka trenutnega simuliranega koraka -- trenutno leto simulacije.
		 */
		int simulationStepNr;
		/**
		 * Rastlinske vrste v simulaciji.
		 */
		std::vector<simulationEntities::CVrstaRastlin> rastlinskeVrste;
		/**
		 * Objekt za določanje življenskih faktorjev rastlinam.
		 */
		LifeFactors lifeFactors;
		/**
		 * Vrne vsebovan objekt za vodenje statistike
		 * @return kazalec na objekt s statistiko
		 */
		statistics::CEntitiesStatistics* getEntitiesStatistics();
		/**
		 * Shrani parametre simulacije:
		 * - lokacije rastlin
		 * - ostale lastnosti posameznih rastlin
		 * - random generatorje
		 * - komulacije rastlin
		 * @param fileName ime datoteke, v katero shranjujemo
		 */
		void saveSimulation(const std::string &fileName);
		/**
		 * Naloži simulacijo iz datoteke
		 * @param fileName ime datoteke, v katero shranjujemo
		 */
		void loadSimulation(const std::string &filename);
		/**
		 * Vrne vsa uporabljana semena za random generator.
		 * @return
		 */
		std::vector<unsigned long long> getRandomGeneratorSeeds();
		/**
		 * Nastavi semena vsem uporabljanim random generatorjem
		 * @param seeds nova semena generatorjev
		 */
		void setRandomGeneratorSeeds(const std::vector<unsigned long long>& seeds);

	private:
		/**
		 * Kreira semena za random generatorje.
		 */
		void createSeeds();
		/**
		 * Shrani statistiko o rastlinah
		 * Tvori log message za čase, koliko kateri del simulacije traja
		 * @param casOstalo čas renderiranja in ostalih rutin
		 * @param casDodajal trajanje "dodaj"
		 * @param casZrastel trajanje "zrasti"
		 * @param casPobijal trajanje "pobij"
		 */
		void saveStatistics(int casOstalo, int casDodajal, int casZrastel, int casPobijal);
		/*!
		* Če simulacija še ni bila inicializirana, kliči tovarno za kreiranje primera nove simulacije
		*/
		void zagotoviInicializiranostSimulacije();

		/*!
		* Vsaki vrsti dodamo nove kandidate in jim določimo pozicije
		* Če bodo pozicije sovpadale z večimi rastlinami, bodo kar takoj odmrle (torej "tu nič ne raste")
		*/
		void dodajRastlineVrstam();

		/*!
		* @hack: izklopili smo ponovno preverjanje dominiranosti, rezultat jemljemo iz prejšnjega
		* simulacijskega koraka
		* poveča starost, velikost, radij, določa moč in komulacije semen VSEM rastlinam
		*/
		void zrastiVseRastlineInDolociMoci();

		/*!
		* Poveča starost, velikost in radij, določa moč in komulacijo semen ENE RASTLINE
		* @param rastlina obravnavana rastlina
		* @param rastlinskaVrsta pripadajoča rastlinska vrsta
		*/
		void zrastiRastlino(simulationEntities::CRastlina& rastlina,
		                    simulationEntities::CVrstaRastlin &rastlinskaVrsta);
		/*!
		* Glede na obdobje zrelosti in starosti, spremeni velikost rastline,
		* tu rastlina ni dominirana, saj funkcijo kličemo le ob nedominiranosti
		* rastlina bolj raste do zrelosti, potem pa še nekaj malega do starosti,
		* dokljer ni velikost enaka 1
		* @param rastlina obravnavana rastlina
		*/
		void povecajVelikostRastline(simulationEntities::CRastlina& rastlina);
		/**
		 * Naj bi izračunala ugodnosti pogojev, ki bi vplivali na hitrost rasti rastline?
		 * @param rastlina rastlina
		 * @return dobljena ugodnost pogojev
		 */
		float izracunajUgodnostPogojev(simulationEntities::CRastlina &rastlina);
		/*!
		* Prodornost naj se ne spreminja: pogoji terena ostajajo vedno enaki
		* @param rastlina obravnavana rastlina
		*/
		void dolociMocRastline(simulationEntities::CRastlina &rastlina);

		/**
		 * Pobije rastline, jih umakne iz simulacije -- tiste, ki trenutno umrejo.
		 */
		void pobijRastline();
		/*!
		* Preveri, ali bo obravnavana rastlina na tem koraku umrla.
		* Če umre, jo odstrani iz 2d mreže rasti rastlin po terenu za ugotavljanje sosednih rastlin.
		* Pobijemo vse od starosti odmrle rastline in pobijemo vse vrste,
		* ki podležejo dominanci drugih rastlin.
		*
		* @param rastlina obravnavana rastlina
		* @return odločitev, ali umre
		*/
		static bool umreSedaj(simulationEntities::CRastlina &rastlina);

		/*!
		* Mapa rastlin na 2D terenu: uporabimo jo za seznam sosednosti,
		* je array vektorjev, ki kažejo na rastline na nekem polju mape.
		*/
		std::vector<simulationEntities::CRastlina*> *rastlinePoTerenu;

		/**
		 * Kazalec na podatke terena: točke in pogoji.
		 */
		data::CTerrainData *terrainData;
		/**
		 * Kazalec na instanco tega razreda, potreben pri sprehodu skozi seznam (remove_if STL).
		 */
		static CEcosystemSimulation *myStaticPtr;
		/**
		 * Največja možna starost rastline, absolutno gledano: maksimalna starost vseh rastlin.
		 */
		int najvecjaMoznaStarostRastline;
		/**
		 * Kazalec na objekt za hranjenje statistike o rastlinskih vrstah in simulaciji nasploh.
		 */
		statistics::CEntitiesStatistics entitiesStatistics;

		/**
		 * Čas trajanja za remove_if pri odstranjevanju rastlin, ki so postale dominirane
		 */
		static int casPobijanjaDominiranihRastlin;
		/**
		 * Timer za merjenje trajanja rutin.
		 */
		QTime timer;
	};
}//namespace ecosystem
#endif
