#ifndef ECOSYSTEM_SIMULATION_ENTITIES_RASTLINA_H
#define ECOSYSTEM_SIMULATION_ENTITIES_RASTLINA_H

#include "tocka.h"
namespace ecosystem {
	class CEcosystemSimulation;
	namespace simulationEntities {
		class CRastlina;
	}//namespace simulationEntities
}//namespace ecosystem

#include "vrstarastlin.h"
#include "tocka2i.h"
#include <vector>

namespace ecosystem {
	namespace simulationEntities {
		/*!
		* \brief lastnosti, faktorji, mo� dominiranost rastline
		*
		* Določa vse o individualni rastlini. Določa, kje in kako bo živela.
		*
		* Hrani podatke o individualni rastlini sami, na primer:
		* - starost,
		* - velikost,
		* - moč in
		* - pozicija.
		* Prav tako hrani lastnosti terena na tem 3D mestu.
		* Določa tudi dominiranost rastline.
		*@author Ale Zamuda (ales.zamuda@uni-mb.si)
		*/
		class CRastlina {
		public:
			/**
			 * Kazalec na vrsto rastline, ki ji ta rastlina pripada.
			 */
			CVrstaRastlin *vrstaRastline;
			/**
			 * Identifikacijska številka rastline. Za vsako novo rastlino
			 * se ta številka poveča. Ko pride do MAX_INT, gre v MIN_INT,
			 * nato spet okoli.
			 */
			int id;

			/**
			 * Trenutna starost rastline, v letih.
			 */
			int trenutnaStarost;
			/**
			 * Trenutna velikost rastline, normalizirana vrednost med 0 in 1.
			 */
			float trenutnaVelikost;
			/**
			 * Ekološki radij rastline v metrih.
			 */
			float trenutniRadij;
			/**
			 * Naslov pozicije na 2D mapi, indeksi so 0-based.
			 */
			Tocka2i pozicijaMap2D;
			/**
			 * Pozicija rastline v 3D prostoru, na mapi, po metrih.
			 */
			drevo::Tocka pozicija3D;
			/*!
			 * Prodornost rastline, vpliva na rast in moč, 0..1
			* = vlažnost terena * potreba rastline po vlagi + ...
			*/
			float prodornost;
			/*!
			 * Moč rastline pri boju z drugimi rastlinami, 0..1
			* = prodornost*trenutnaVelikost
			*/
			float trenutnaMoc;

			/**
			 * Strmina terena na lokaciji te rastline: 0..1.
			 */
			float strminaTukaj;
			/**
			 * Vlažnost terena na lokaciji te rastline, 0..1.
			 */
			float vlaznostTukaj;
			/**
			 * Osončenost terena na lokaciji te rastline, 0..1.
			 */
			float sonceTukaj;
			/**
			 * Vetrovnost terena na lokaciji te rastline, 0..1.
			 */
			float veterTukaj;
			/**
			 * Nadmorska višina terena na lokaciji te rastline, 0..1.
			 */
			float visinaTukajNormNadVodo;
			/**
			 * Določen korak (število let pri katerih) ko rastlina umre.
			 */
			float umreNaKoraku;
			/**
			 * Določilo, ali je v trenutnem koraku rastlina dominirana, vpliva na rast in
			 * določanje odmiranja.
			 */
			bool jeDominirana;

			/*!
			* pripravi parametre nove rastline:
			*	- izbere 2d pozicijo na mapi, glede na to, ali naj jo postavi blizu ali kjerkoli
			*	- shrani rastlino v mapi 2d pozicij rastlin za iskanje sosedov
			*	- glede na to 2d pozicijo dolo� 3d pozicijo in vlanost
			*	- določi prodornost rastline in strmino, glede na 2D pozicijo
			*	- ostale parametre da na 0
			* @param vrstaRastlin pripadajo� vrsta rastlin
			* @param pozicija2d lokacija rastline (krpa)
			* @param pozicija3d 3d lokacija rastline v svetu
			* @return
			*/
			CRastlina(CVrstaRastlin &vrstaRastlin, const Tocka2i &pozicija2d, const drevo::Tocka &pozicija3d);
			/*!
			* če je v preseku in je sovražna rastlina močneja, je ta inferiorna
			* @param rastlinaSovraznik povezava na sovražno rastlino
			* @return
			*/
			bool presekaRadijRastline(CRastlina &rastlinaSovraznik);

		private:
			/*!
			* vodi seznam potencialnih sosedov:
			* 1-ko dodamo rastlino, napolnimo tudi ta seznem
			*   -pri tem jemljemo rastline iz okolice 2d mape, za največji možen radij
			*/
			static std::vector<CRastlina*> potencialniSosedi;

			/*!
			* določi normalizirano višino nad vodo, vrednost 0..1, 1=najvišje
			*/
			void dolociProdornostRastline();

			/**
			 * Določi višino nad vodo, tako da poišče podatke iz terrainData.
			 * @return višina na mestu rasti rastline
			 */
			float dolociVisinoTukajNadVodoNormalizirano();
			/**
			 * Določi strmino terena na mestu rasti.
			 * @param pozicijaMap2D indeks krpe, na kateri rastlina raste
			 * @return strmina na krpi
			 */
			float getStrminaTerena(Tocka2i &pozicijaMap2D);
			/**
			 * Določi vlažnost na mestu rastline.
			 * @param pozicijaMap2D indeks krpe, na kateri rastlina raste
			 * @return vlažnost na krpi
			 */
			float dolociVlaznostNoveRastline(Tocka2i &pozicijaMap2D);
			/*!
			* Umre z enako verjetnostjo na kateremkoli delu časa med starostjo in sigurno smrtjo,
			 * tu določi korak, kdaj naj umre.
			*/
			void dolociSmrt();

			friend class ecosystem::CEcosystemSimulation;
			/**
			 * Random seme za generiranje časov smrti.
			 */
			static unsigned long long randomCasSmrti;
			/**
			 * Identifikator za prirejanje ID-jev rastlinam.
			 * Ta hrani zadnji generiran ID. Prva rastlina bo imela ID=1.
			 */
			static int lastID;
		};
	}//namespace simulationEntities
}//namespace ecosystem
#endif // ECOSYSTEM_SIMULATION_ENTITIES_RASTLINA_H
