#ifndef ECOSYSTEM_SIMULATION_ENTITIES_VRSTA_RASTLIN_H
#define ECOSYSTEM_SIMULATION_ENTITIES_VRSTA_RASTLIN_H

#include <list>
#include <vector>

namespace drevo {
	class Tocka;
}
namespace ecosystem {
	class Tocka2i;
	namespace simulationEntities {
		class CVrstaRastlin;
	}//namespace simulationEntities
	namespace data {
		class CTerrainData;
	}
}//namespace ecosystem

#include "rastlina.h"
#include "pogoj.h"
#include "entitiesstatistics.h"
#include "tocka2i.h"
#include "tocka.h"

namespace ecosystem {
	/*!
	* \namespace ecosystem::simulationEntities
	* \brief hrani simulacijske entitete, ki vplivajo na simulacijo.
	*
	* Na simulacijo vplivajo že pogoji okolja, vendar entitete v simulaciji
	* vplivajo tudi ena na drugo. Trenutno so te entitete posamezne rastline in
	* rastlinske vrste:
	* - neka rastlinska vrsta lahko prevlada nad drugo, ker ima več semen na terenu in
	* - rastlina lahko nadvlada nad drugo rastlino, če jo dominira.
	*
	* Rastlinske vrste kot tudi rastline so tako entitete, ki jih obravnavamo v tem
	* paketu. Pri vrstah hranimo splone parametre simulacije, ki so ponavadi dobljeni
	* pred pričetkom simulacije (npr. hitrost rasti), pri samih rastlinah pa vodimo
	* dejanske parametre individualnih rastlin in parametre okolja na točko tistem mestu,
	* kjer posamezna rastlina raste.
	*/
	namespace simulationEntities {
		/*!
		* \brief pogoji, zrelost, semena vrste
		*
		* Obravnava vrste rastlin.
		* Vsebuje podatke o vrstah rastlin:
		* - želene lastnosti pogojev
		* - verjetnostne parametre
		*
		* Rastline pripadajo rastlinskim vrstam, ki dajejo splošne lastnosti posameznim
		* rastlinam. Tako tu nastavimo razne potrebe in verjetnosti, ki jih upoštevamo
		* pri individualnih rastlinah.
		*
		* \sa CRastlina
		* @author Aleš Zamuda (ales.zamuda@uni-mb.si)
		*/
		class CVrstaRastlin {
		public:
			/**
			 * Tip vrste rastline. Uporabljen za določanje, kateri parametri proceduralnega modela
			 * naj se uporabijo za ta tip rastlin, saj lahko tako oštevilčimo vrsto rastline.
			 */
			int tip;
			/**
			 * Število novih primerkov rastlin te vrste v vsakem simulacijskem koraku.
			 */
			float stNovihRastlinNaSimulacijskiKorak;
			/*!
			* relativna velikost, ko jo drevo doseže, ko lahko prvič seje semena
			*/
			float razmerjeMedVelikostjoInRadijemKrosnje;
			/**
			 * Delež rastlin, ki raste blizu svoje vrste, 0..1. Ostale zrastejo
			 * kjerkoli na terenu, kjer je dovoljeno (tam, kjer ni vode).
			 */
			float delezRastiBlizuSvojeVrste;
			/**
			 * Velikost rastline, ko dozori, 0..1.
			 */
			float velikostObZrelosti;

			/**
			 * Čas 0..1, ko rastlina te vrste dozori in lahko daje prva semena.
			 * zrelost = prva semena, počasnejša rast
			 */
			int casZrelosti;
			/**
			 * Čas, ko se rastlina te vrste postara, da ne raste več, 0..1.
			 */
			int casStarosti;
			/**
			 * Maksimalna starost te vrste rastlin, 0..1.
			 */
			int casSigurneSmrti;

			/** Če kdo drug dominira rastlino te vrste, umre s to verjetnostjo (0..1).
			 */
			float verjetnostUmrtjaObDominiranosti;
			/**
			 * Škodljivost zaradi neugodnega naklona, 0..1.
			 */
			float skodljivostNaklona;
			/**
			 * Vpliv moči pri merjenju moči, ali rastlina te rastlinske vrste premaga
			 * drugo rastlino, npr. grmovje je šibkejše napram smrekam.
			 */
			float vplivMoci;
			/**
			 * Pogoj, ki opisuje zahteve rastlin te vrste za višino.
			 */
			CPogoj visina;
			/**
			 * Pogoj, ki opisuje zahteve rastlin te vrste za vlago.
			 */
			CPogoj vlaga;
			/**
			 * Pogoj, ki opisuje zahteve rastlin te vrste za sonce.
			 */
			CPogoj sonce;
			/**
			 * Pogoj, ki opisuje zahteve rastlin te vrste za veter.
			 */
			CPogoj veter;

			/// ////////////////////////////////////////////////////////////////////
			/// parametri, ki se izračunajo iz zgornjih:
			/// ////////////////////////////////////////////////////////////////////
			/**
			 * Izračunan parameter, kolikšna je komulacija semen za to vrsto.
			 */
			float komulacijaSemen;
			/**
			 * Izračunan parameter,
			 * če je stNovihRastlinNaSimulacijskiKorak<1, se tu prišteva, dokljer ni >=1
			 */
			float komulacijaPrihajajocihRastlin;
			/**
			 * Izračunan parameter, ki določa maksimalno število iskanj rastline za dominiranje.
			 */
			static int maxStKvadratovNajdemRadijSoseda;

			/**
			 * Kazalec na podatke o terenu, za dodajanje novih primerkov na površino terena.
			 */
			data::CTerrainData *terrainData;
			/**
			 * Seznam vseh živih primerkov rastlin te vrste, pomaga pri upodabljanju in
			 * kreiranju novih rastlin.
			 */
			std::list<CRastlina> ziviPrimerkiTeVrste;
			/**
			 * Definicija tipa za iterator seznama za žive primerke te vrste.
			 */
			typedef std::list<CRastlina>::iterator ziviPrimerkiTeVrsteIter;
			/**
			 * Ustvari in inicializira rastlinsko vrsto.
			 *
			 * @param terrainData terenski podatki, s točkami in ekološkimi podatki
			 * @param entitiesStatistics shramba statistike
			 * @param tip tip rastlinske vrste, olajša preslikavo za uporabo pri proceduralnem modelu drevesa
			 * @param stNovihRastlinNaSimulacijskiKorak število novih rastlin, ki jih dodamo v enem letu
			 * @param komulacijaPrihajajocihRastlin število rastlin, ki naj zraste v prvem koraku simulacije
			 * @param casZrelosti čas 0..1, ko rastlina dozori, začne dajati semena
			 * @param casStarosti čas 0..1, ko rastlina doseže starost (dokončno zraste)
			 * @param casSigurneSmrti čas, ko vse rastline te vrste sigirno umrejo, 0..1
			 * @param velikostObZrelosti velikost rastline ob zrelosti, 0..1
			 * @param verjetnostUmrtjaObDominiranosti verjetnost umrtja rastline, če postane dominirana 0..1
			 * @param potrebaPoVisini potreba rastline po določenem pasu višine, optimalna višina 0..1
			 * @param tolerantnostSpremembVisine tolerirana sprememba glede na optimalno višino 0..1
			 * @param verjetnostSmrtiObNeugodniVisini verjetnost 0..1, da rastlina ob neugodni višini odmre
			 * @param slabVplivVisineNaHitrostRasti slab vpliv višine 0..1 na hitrost rasti
			 * @param potrebaPoVlagi potreba po vlagi 0..1, optimalna vrednost
			 * @param tolerantnostSpremembVlage toleranca odstopanja 0..1 od optimalne vlage, da je vlaga še ugodna
			 * @param verjetnostSmrtiObNeugodniVlagi verjentost 0..1, da rastlina umre ob neugodnem pogoju vlage
			 * @param slabVplivVlageNaHitrostRasti slab vpliv vlage na hitrost rasti, 0..1
			 * @param potrebaPoSoncu potreba rastline po soncu, 0..1
			 * @param tolerantnostSpremembSonca toleriranost sprememb sonca, 0..1
			 * @param verjetnostSmrtiObNeugodnemSoncu verjetnost smrti ob neugodnem pogoju sonca, 0..1
			 * @param slabVplivSoncaNaHitrostRasti slab vpliv osončenosti na hitrost rasti, 0..1
			 * @param potrebaPoVetru optimalna vrednot vetrovnosti, 0..1
			 * @param tolerantnostSpremembVetra tolerirana sprememba vetrovnosti, da je še dopustna, 0..1
			 * @param verjetnostSmrtiObNeugodnemVetru verjetnost smrti, če je pogoj vetrovnosti neugoden, 0..1
			 * @param slabVplivVetraNaHitrostRasti slab vpliv, če je pogoj vetrovnosti za rastlino neugoden
			 * @param razmerjeMedVelikostjoInRadijemKrosnje razmerje med velikostjo in radijem krošnje (metri/metri)
			 * @param delezRastiBlizuSvojeVrste odstotek rastlin, ki zraste blizu svoje vrste, 0..1
			 * @param skodljivostNaklona škodljivost naklona, 0..1
			 * @param vplivMoci vpliv moči pri primerjanju moči različnih rastlinskih vrst, grm je šibkejši od hrasta
			 * @return
			 */
			CVrstaRastlin(data::CTerrainData *terrainData,
			              statistics::CEntitiesStatistics *entitiesStatistics,
			              int tip,
			              float stNovihRastlinNaSimulacijskiKorak,
			              float komulacijaPrihajajocihRastlin,
			              int casZrelosti,
			              int casStarosti,
			              int casSigurneSmrti,
			              float velikostObZrelosti,
			              float verjetnostUmrtjaObDominiranosti,

			              float potrebaPoVisini,
			              float tolerantnostSpremembVisine,
			              float verjetnostSmrtiObNeugodniVisini,
			              float slabVplivVisineNaHitrostRasti,

			              float potrebaPoVlagi,
			              float tolerantnostSpremembVlage,
			              float verjetnostSmrtiObNeugodniVlagi,
			              float slabVplivVlageNaHitrostRasti,

			              float potrebaPoSoncu,
			              float tolerantnostSpremembSonca,
			              float verjetnostSmrtiObNeugodnemSoncu,
			              float slabVplivSoncaNaHitrostRasti,

			              float potrebaPoVetru,
			              float tolerantnostSpremembVetra,
			              float verjetnostSmrtiObNeugodnemVetru,
			              float slabVplivVetraNaHitrostRasti,

			              float razmerjeMedVelikostjoInRadijemKrosnje,
			              float delezRastiBlizuSvojeVrste,
			              float skodljivostNaklona,
			              float vplivMoci);

			/**
			 * Doda komulirane rastline, kreira primerke.
			 * Tu bi bilo možno omejiti število rastlin, ki se maksimalno doda v eni generaciji.
			 *
			 * @param dodanaKomulacija komulacija (vsota) novih rastlin
			 * @param rastlinePoTerenu povezava na seznam obstoječih rastlin
			 */
			void dodajKomuliraneRastline(float dodanaKomulacija, std::vector<CRastlina*> *rastlinePoTerenu);
			/**
			 * Doda nov primerek v to rastlinsko vrsto in ga inicializira.
			 * @param rastlinePoTerenu seznam rastlin te rastlinske vrste
			 * @param pozicija2d pozicija 2d (indeks krpe) na terenu
			 * @param pozicija3d pozicija 3d (na površju terena)
			 */
			void dodajNovPrimerek(std::vector<CRastlina*> *rastlinePoTerenu, const Tocka2i &pozicija2d, const drevo::Tocka &pozicija3d);
			/**
			 * Kazalec na objekt za hranjenje statistike o rastlinskih vrstah in simulaciji nasploh.
			 */
			statistics::CEntitiesStatistics *entitiesStatistics;
		};
	}//namespace simulationEntities
}//namespace ecosystem
#endif // ECOSYSTEM_SIMULATION_ENTITIES_VRSTA_RASTLIN_H
