#ifndef ECOSYSTEM_LIFE_FACTORS_H
#define ECOSYSTEM_LIFE_FACTORS_H

#include "terraindata.h"
#include "vrstarastlin.h"
#include "rastlina.h"

namespace ecosystem {
	/*!
	 * \brief življenski faktorji rastline
	 *
	 * Upotevamo:
	 * - vse rastline umrejo, ko dosežejo točno določeno starost
	 * - rojevajo se fiksno
	 * - rast je fiksna
	 * - rastlini se prekrivata, če sta znotraj sosečin (enakost tudi teje)
	 *
	 *@author Ale Zamuda (ales.zamuda@uni-mb.si)
	 */
	class LifeFactors {
	public:
		/**
		 * Shrani pripadajočo rastlinsko vrsto in inicializira random generatorje.
		 * @param rastlinskeVrste
		 * @return
		 */
		LifeFactors(std::vector<simulationEntities::CVrstaRastlin> *rastlinskeVrste);

		/*!
		 * Preverimo faktorje, ki vplivajo, da obravnavana rastlina umre.
		 * Najprej preverimo pomembneje faktorje, da je hitreje.
		 * @param rastlina obravnavana
		 * @return rastlina umre
		 */
		bool umreSedaj(simulationEntities::CRastlina &rastlina);
		/*!
		 * Shrani kazalec na polje vektorjev kazalcev rastlin po pozicijah na mapi
		 * @param rastlinePoTerenu
		 */
		void setRastlinePoTerenu(std::vector<simulationEntities::CRastlina*> *rastlinePoTerenu);
		/*!
		 * v kvadratih radija, kjer bi lahko bil
		 * TODO: dominiranje deluje e vedno O(N^2), vseeno pa very fast
		 * @param rastlina
		 * @return
		 */
		inline bool rastlinaJeDominirana(simulationEntities::CRastlina &rastlina);

	private:
		friend class CEcosystemSimulation;/// za pobiranje random generatorjev
		/**
		 * Seme za random generator.
		 */
		/// unsigned long long randomSmrtiZaradiStarosti; // zamenjan z randomCasSmrti
		/**
		 * Seme za random generator.
		 */
		unsigned long long randomSmrtiZaradiDominiranosti;
		/**
		 * Seme za random generator.
		 */
		unsigned long long randomSmrtiZaradiNeugodnostiVisine;
		/**
		 * Seme za random generator.
		 */
		unsigned long long randomSmrtiZaradiNeugodnostiVlage;
		/**
		 * Seme za random generator.
		 */
		unsigned long long randomSmrtiZaradiNeugodnostiSonca;
		/**
		 * Seme za random generator.
		 */
		unsigned long long randomSmrtiZaradiNeugodnostiVetra;
		/**
		 * Seme za random generator.
		 */
		unsigned long long randomSmrtiZaradiNeugodnostiNaklona;

		/**
		 * Hrani array seznamov za rastlinske vrste, s svojimi rastlinami po terenu.
		 *
		 * Pointer na array vektorjev pointerjev:
		 * pointer na array [array vsebuje vektorje, vektorji hranijo pointerje na rastline]
		 */
		std::vector<simulationEntities::CRastlina*> *rastlinePoTerenu;

		/**
		 * Rastlinske vrste na terenu.
		 */
		std::vector<simulationEntities::CVrstaRastlin> *rastlinskeVrste;

		/**
		 * Določi, ali podana rastlina v tem koraku umre zaradi starosti.
		 * @param rastlina podana rastlina
		 * @return ali umre (jo odstranimo)
		 */
		bool umreZaradiStarosti(simulationEntities::CRastlina &rastlina);
		/**
		 * Določi, ali podana rastlina v tem koraku umre zaradi šibkosti.
		 * @param rastlina podana rastlina
		 * @return ali umre (jo odstranimo)
		 */
		bool umreZaradiSibkosti(simulationEntities::CRastlina &rastlina);

		/**
		 * Določi, ali podana rastlina v tem koraku umre zaradi neugodnosti višine.
		 * @param rastlina podana rastlina
		 * @return ali umre (jo odstranimo)
		 */
		bool umreZaradiVisine(simulationEntities::CRastlina &rastlina);
		/**
		 * Določi, ali podana rastlina v tem koraku umre zaradi neugodnosti vlage.
		 * @param rastlina podana rastlina
		 * @return ali umre (jo odstranimo)
		 */
		bool umreZaradiNeugodneVlage(simulationEntities::CRastlina &rastlina);
		/**
		 * Določi, ali podana rastlina v tem koraku umre zaradi neugodnosti osončenja.
		 * @param rastlina podana rastlina
		 * @return ali umre (jo odstranimo)
		 */
		bool umreZaradiNeugodnegaSonca(simulationEntities::CRastlina &rastlina);
		/**
		 * Določi, ali podana rastlina v tem koraku umre zaradi neugodnosti vetra.
		 * @param rastlina podana rastlina
		 * @return ali umre (jo odstranimo)
		 */
		bool umreZaradiNeugodnegaVetra(simulationEntities::CRastlina &rastlina);
		/**
		 * Določi, ali podana rastlina v tem koraku umre zaradi neugodnosti strmine.
		 * @param rastlina podana rastlina
		 * @return ali umre (jo odstranimo)
		 */
		bool umreZaradiStrmine_oz_Prsti_oz_kislostiTal_Ph(simulationEntities::CRastlina &rastlina);
		/**
		 * Določi, ali podana rastlina v tem koraku umre zaradi dominiranosti.
		 * @param rastlina podana rastlina
		 * @return ali umre (jo odstranimo)
		 */
		bool umreZaradiDominiranosti(simulationEntities::CRastlina &rastlina);

		/*!
		 * Preiče podano polje za obstajajočimi rastlinami na njih in vrne, ali se podana rastlina seka
		 * @param startPoint prva krpa (ik)
		 * @param endPoint zadnja krpa (ik)
		 * @param rastlina dana rastlina
		 * @return ali se seka z moènejo
		 */
		inline bool obstajaMocnejsiSovraznikNaPoljuSosescine(Tocka2i &startPoint,
		        Tocka2i &endPoint, simulationEntities::CRastlina &rastlina);
	};
}//namespace ecosystem
#endif
