#ifndef ECOSYSTEM_SEJALEC_RASTLIN_NA_TEREN_H
#define ECOSYSTEM_SEJALEC_RASTLIN_NA_TEREN_H

#include "tocka.h"
#include "tocka2i.h"
#include "sejaleckjerkolirastlinnateren.h"
#include "terraindata.h"
#include "vrstarastlin.h"

namespace ecosystem {
	/*!
	* \brief postavljanje rastlin v bliino starev
	* Razred reši iskanje rastlin po seznamu ob dodajanju nove rastline v
	* vnaprejnjo določitev indeksov bližnjih rastlin, nakar se samo enkrat sprehodimo
	* skozi vse obstoječe rastline in dodamo nove rastline.
	* @author Aleš Zamuda (ales.zamuda@uni-mb.si)
	*/
	class CSejalecRastlinNaTeren {
	public:
		/**
		 * Kreira sejalca rastlin podane vrste (seje samo to vrsto!).
		 * Rastline sam poseje v bližini obstoječih rastlin ali pa delo
		 * delegira objektu "sejalecKjerkoliRastlinNaTeren", ki rastline
		 * poseje poljubno, kjerkoli.
		 *
		 * @param terrainData povezava na podatke o terenu
		 * @param pozicije2d pozicije 2d za to vrsto rastlin, za pregled obstoječih
		 * @param pozicije3d pozicije 23 za to vrsto rastlin, kamor shrani novo rastlino
		 * @param vrstaRastlin vrsta rastlin, ki jo bo sejal
		 * @return
		 */
		CSejalecRastlinNaTeren(data::CTerrainData &terrainData, std::vector<Tocka2i> &pozicije2d, std::vector<drevo::Tocka> &pozicije3d, simulationEntities::CVrstaRastlin &vrstaRastlin);

		/*!
		* Določi pozicije za VSAKO novo rastlino te vrste na tem koraku.
		* prednost: samo enkrat gremo skozi seznam rastlin te vrste, zato zahtevnost zmanjšamo na O(n)...
		* algoritem:
		* A. rast blizu
		*  1)kreiramo indekse,
		*	2)indekse sortiramo
		*  3)gremo povrsti po rastlinah, primerjamo indekse rastlin s kreiranimi indeksi
		*  4)če je indeks enak, kličemo kreiranje nove rastline. več indeksov je lahko
		*    enakih (seznam lahko vsebuje enake indekse)
		*  5)na koncu za vsako rastlino določimo še 3d pozicijo. če sta torej 2d in
		*    3d pozicija ok, šele tedaj dodamo rastlino  v seznam.
		* B. rast kjerkoli
		*  1)sproti vodimo rastline, ki lahko rastejo kjerkoli in jih generiramo na koncu
		*  2)te rastline dodamo v seznam 2d in 3d, za nad vodo preverjamo samo 3d pozicijo
		*/
		void dolociPozicijeRastlin();
		/*!
		* Kreira indekse rastlin, ki so stari rastočih rastlin blizu njih.
		* Za rastline, ki ne rastejo blizu, vrne število takih izjem.
		* @param indeksiBlizu seznam rastočih blizu
		* @param stRastlinRastocihKjerkoli število rastočih kjerkoli
		*/
		void kreirajIndekseRastlinBlizu(std::vector<int> &indeksiBlizu, int &stRastlinRastocihKjerkoli);
		/*!
		* vrne, ali trenutna rastlina naj raste blizu, ali naj raste kjerkoli: po verjetnosti
		* za novo rastlino določimo naključno mesto (indeks) na terenu
		* verjetnost rasti nove rastline na nekem območju je zaenkrat enakomerno porazdeljena
		* izboljšava:
		* - da bo nova rastlina zrastla blizu prejšnje (prenos semen!)
		* @return
		*/
		bool rasteBlizu();
		/*!
		* Doda rastline blizu.
		* Če bo seznam prazen, ne bomo generirali pač nobene rastline.
		* @param indeksiBlizu indeksi rastlin, ki naj jih dodamo blizu
		*/
		void dodajRastlineBlizu(std::vector<int> &indeksiBlizu);
		/*!
		*poiščemo neko svojo rastlino in zrastemo 1 polje stran od nje...? :)
		*bližnja vedno obstaja, saj dobimo točko od prej najdene rastline blizu
		* @param pozicija2dObstojeceRastline pozicija obstoječe rastline
		* @param pozicija2dNova dobljena nova pozicija nove rastline, ki raste blizu stare
		*/
		void dolociPozicijoNoveRastline2Dblizu(Tocka2i &pozicija2dObstojeceRastline, Tocka2i &pozicija2dNova);
		/**
		 * Opravi test, ali je predlagan položaj nad vodo (na osnovi 3D položaja)
		 * in če je, novo (podano) rastlino doda v seznam obstoječih rastlin, shrani jo
		 * v pozicije2d in pozicije3d.
		 * @param pozicija2d predlagan položaj za rastlino
		 * @param pozicija3d 3D položaj za rastlino
		 */
		void dodajRastlinoVSeznama(const Tocka2i &pozicija2d, const drevo::Tocka &pozicija3d);

	private:
		/**
		 * Kazalec na obstoječe podatke o terenu, potrebujem za točke terena.
		 */
		data::CTerrainData &terrainData;
		/**
		 * Pozicije obstoječih rastlin po krpah terena, za vrsto, ki jo sejemo.
		 */
		std::vector<Tocka2i> &pozicije2d;
		/**
		 * Pozicije rastlin po terenu, 3D lokacije vrste, za vrsto, katero sejemo.
		 */
		std::vector<drevo::Tocka> &pozicije3d;
		/**
		 * Pripadajoča vrsta rastlin, ki jo sejemo.
		 */
		simulationEntities::CVrstaRastlin &vrstaRastlin;

		/**
		 * Objekt, ki poseje rastlino kjerkoli drugje na terenu, ne nujno blizu.
		 */
		CSejalecKjerkoliRastlinNaTeren sejalecKjerkoliRastlinNaTeren;

		/**
		 * Seme za random generator.
		 */
		static unsigned long long randomRastiBlizu;
		/**
		 * Seme za random generator.
		 */
		static unsigned long long randomIzbireBliznjeRastline;
		/**
		 * Seme za random generator.
		 */
		static unsigned long long randomIzbireStraniRastiObBliznjiRastlini;
		friend class CEcosystemSimulation;/// za pobiranje random generatorjev
	};
}//namespace ecosystem
#endif // ECOSYSTEM_SEJALEC_RASTLIN_NA_TEREN_H
