/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GUI_ACTIONHANDLINGDESIGNHANDLER_H
#define GUI_ACTIONHANDLINGDESIGNHANDLER_H

#include <QObject>
#include <vector>

namespace drevo {
	class Krivulja;
}

namespace ecosystem {
	namespace render {
		class EcosystemRender;
	}
}

namespace gui {
	class DistributionDesigner;
	/*!
	 * \namespace gui::actionhandling
	 * \brief skrbi za obravnavo uporabnikovih akcij
	 *
	 * MainWindow za obdelovanje uporabnikovih akcij uporablja razrede v imenskem
	 * prostoru actionhandling: DesignHandler skrbi za načrtovanje grafov,
	 * FileHandler za delo z datotekami, NavigationHandler za postavljanje kamere,
	 * TimeKeeper za poganjanje timerjev animacije, ViewHandler za spreminjanje
	 * načinov pogleda na drevesa (ne pa tudi kamere).
	 *
	 * @author Ales Zamuda
	 */
	namespace actionhandling {

		/**
		 * Obravnava akcije za načrtovanje parametrov drevesa, ki so
		 * določeni z grafi.
		 *
		 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
		*/
		class DesignHandler : public QObject {
			Q_OBJECT
		public:
			/**
			 * Ustvari objekt, shrani pripadajoči model.
			 *
			 * @param parent očetovski objekt
			 * @param parentWindow vrhnje okno
			 * @param model model, ki ga ureja
			 */
			DesignHandler(QObject *parent, QWidget *parentWindow, ecosystem::render::EcosystemRender * model);

			/**
			 * Pobriše objekt
			 */
			~DesignHandler();

		signals:
			/**
			 * Sproži signal, da naj se model znova izriše
			 */
			void forceRepaint();

		public slots:
			/**
			 * Prikaže dialog za oblikovanje grafov porazdelitve žil.
			 */
			void actionDesign_strand_distribution();
			/**
			 * Prikaže dialog za oblikovanje grafov kotov vejitev.
			 */
			void actionDesign_branching_angles();
			/**
			 * Prikaže dialog za oblikovanje grafov minimalne absolutne dolžine
			 */
			void actionDesign_branch_absolute_minimal_length();
			/**
			 * Prikaže dialog za oblikovanje grafov maksimalne absolutne dolžine
			 */
			void actionDesign_branch_absolute_maximal_length();
			/**
			 * Prikaže dialog za oblikovanje grafov relativne dolžine.
			 */
			void actionDesign_branch_lenght_scaling();
			/**
			 * Prikaže dialog za oblikovanje grafov relativne dolžine.
			 */
			void actionDesign_gravimorphism_impact_distribution();
			/**
			 * Prikaže dialog za urejanje naprednih parametrov drevesa.
			 */
			void actionAdvanced_tree_properties();

			/**
			 * Shrani urejeno krivuljo (apply) v model.
			 * @param value1 tipi parametrov
			 * @param value2 seznem krivulj po g
			 * @param value3 seznem krivulj po w
			 */
			void updateKrivulja(const std::vector<int>&, const std::vector<drevo::Krivulja*>&,
								const std::vector<drevo::Krivulja*>&);
			/**
			 * Propagira hover po G v Drevo.
			 * @param indexMin od katerega G dalje
			 * @param indexMax do katerega G
			 */
			void applyHoverGChanged(int indexMin, int indexMax);
			/**
			 * Propagira hover po W v Drevo.
			 * @param indexMin od katerega W dalje
			 * @param indexMax do katerega W
			 */
			void applyHoverWChanged(int indexMin, int indexMax);

		private:
			/**
			 * Kreira dialog za dizajniranje porazdelitev.
			 * @param tipParametra tip parametra
			 * @param porazdelitevG graf porazdelitve po G
			 * @param porazdelitevW graf porazdelitve po W
			 * @param labeleG labele za graf G po y
			 * @param labeleW labele za graf W po y
			 * @param title append za naslov okna (izpiše, kateri tip parametra urejamo)
			 */
			void createDesigner(const std::vector<int>&tipParametra, const std::vector<drevo::Krivulja*>&porazdelitevG,
								const std::vector<drevo::Krivulja*>&porazdelitevW, const QString *labeleG[], const QString *labeleW[],
								const QString &title);
			/**
			 * Vrhnje okno.
			 */
			QWidget *parentWindow;

			/**
			 * Model, ki ga urejamo
			 */
			ecosystem::render::EcosystemRender * model;
		};

	}

}

#endif
