/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef DISTRIBUTIONDESIGNER_H
#define DISTRIBUTIONDESIGNER_H

#include "ui_distributiondesigner.h"
#include <QPolygonF>
#include <QRectF>
#include <vector>
namespace drevo {
	class Krivulja;
}

namespace gui {
	/**
	 * Skrbi za nastavljanje parametrov drevesa, ki so odvisni od indeksov G in W.
	 * Prikaže dialog z dvema grafoma, nastavi točke krivulj po G in W, ter sporoča spremembe grafa.
	 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
	 */
	class DistributionDesigner: public QDialog, private Ui::DistributionDesigner {
		Q_OBJECT
	public:
		/**
		 * Ustvari dialog in poveže slote z grafi.
		 *
		 * @param parent starševski objekt, ki me zbriše
		 * @param tipParametra tip parametra, ki ga urejamo
		 * @param porazdelitevG točke krivulje porazdelitve G
		 * @param porazdelitevW točke krivulje porazdelitve W
		 * @param labeleG labele za graf G po y
		 * @param labeleW labele za graf W po y
		 * @param title append za naslov okna (izpiše, kateri tip parametra urejamo)
		 */
		DistributionDesigner( QWidget *parent, std::vector<int> tipParametra, std::vector<drevo::Krivulja*> porazdelitevG, std::vector<drevo::Krivulja*> porazdelitevW,
							  const QString *labeleG[], const QString *labeleW[], const QString &title);
		/**
		 * Pobriše undo krivulje.
		 */
		~DistributionDesigner();
		/**
		 * Vrne obkrožajoči kvader iz hoverpoints grafa G
		 * @return obkrožajoči kvader za G
		 */
		QRectF boundingRectG() const;
		/**
		 * Vrne obkrožajoči kvader iz hoverpoints grafa W
		 * @return obkrožajoči kvader za W
		 */
		QRectF boundingRectW() const;

	signals:
		/**
		 * Točke grafa so se spremenile
		 * @param tocke tipi parametrov krivulj
		 * @param krivuljaG nove krivulje po G
		 * @param krivuljaW nove krivulje po W
		 */
		void pointsUpdated(const std::vector<int>&, const std::vector<drevo::Krivulja*>&,
						   const std::vector<drevo::Krivulja*>&);
		/**
		 * Hover po G se je spremenil
		 * @param indexMin indeks, od katerega začnem hover
		 * @param indexMax indeks, do katerega končam hover
		 */
		void hoverGChanged(int indexMin, int indexMax);
		/**
		 * Hover po W se je spremenil
		 * @param indexMin indeks, od katerega začnem hover
		 * @param indexMax indeks, do katerega končam hover
		 */
		void hoverWChanged(int indexMin, int indexMax);

	public slots:
		/**
		 * Pretvori podane krivulje v točke narisanih grafov v widgetih
		 */
		void setKrivulje();
		/**
		 * Klican, ko hoverpoints G spremeni indeks, da mi pošljemo ven celoštevilski indeks
		 * @param point kliknjena točka na widgetu
		 */
		void applyHoverGChanged(const QPointF &point);
		/**
		 * Klican, ko hoverpoints W spremeni indeks, da mi pošljemo ven celoštevilski indeks
		 * @param point kliknjena točka na widgetu
		 */
		void applyHoverWChanged(const QPointF &point);
		/**
		 * Sprožena, ko kliknemo cancel, ponastavimo parametre.
		 */
		void ponastaviParametreDrevesa();

	protected slots:
		/**
		 * Sprožen ob spremembi točk hoverpoints-ov, za osvežitev parametrov v OpenGL
		 */
		void fireKrivuljeChanged();

	private:
		/**
		 * Točke iz grafa G v widgetu pretvori v zapis Krivulja.
		 */
		std::vector<drevo::Krivulja*> createKrivuljeG() const;
		/**
		 * Točke iz grafa W v widgetu pretvori v zapis Krivulja.
		 */
		std::vector<drevo::Krivulja*> createKrivuljeW() const;
		/**
		 * Seznam urejanih parametrov
		 */
		std::vector<int> tipParametra;
		/**
		 * Urejana porazdelitev po G.
		 */
		std::vector<drevo::Krivulja*> porazdelitevG;
		/**
		 * Urejana porazdelitev po W.
		 */
		std::vector<drevo::Krivulja*> porazdelitevW;
		/**
		 * Undo za urejano porazdelitev po G.
		 */
		std::vector<drevo::Krivulja*> porazdelitevGUndo;
		/**
		 * Undo za urejano porazdelitev po W.
		 */
		std::vector<drevo::Krivulja*> porazdelitevWUndo;
	};
}
#endif
