/****************************************************************************
**
** Copyright (C) 2005-2006 Trolltech AS. All rights reserved.
**
** This file is part of the demonstration applications of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef HOVERPOINTS_H
#define HOVERPOINTS_H

#include <QtGui>

namespace gui {
	class QBypassWidget;

	/**
	 * Razred za risanje točk grafa na kontrole.
	 * Lovi evente, kot so pritisk miškinih gimbov, osvežitev widget-a,..
	 *
	 * Ko uporabnik klikne na točko, jo lahko premika. Točke so shranjene
	 * v pozicije v pikslih, vendar se preračunajo ob resize.
	 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
	 */
	class HoverPoints : public QObject {
		Q_OBJECT
	public:
		/**
		 * Tip, kako se narišejo kontrolne točke krivulje: krog ali kvadrat.
		 */
		enum PointShape {
		    CircleShape,
		    RectangleShape
		};

		/**
		 * Pribitje točke levo/desno/gor/dol.
		 */
		enum LockType {
		    LockToLeft = 0x01,
		    LockToRight = 0x02,
		    LockToTop = 0x04,
		    LockToBottom = 0x08
		};

		/**
		 * Sortiranje točk.
		 */
		enum SortType {
		    NoSort,
		    XSort,
		    YSort
		};

		/**
		 * Tip povezav med kontrolnimi točkami: brez, daljice, krivulje (B-)
		 */
		enum ConnectionType {
		    NoConnection,
		    LineConnection,
		    CurveConnection
		};

		/**
		 * Ustvari objekt, shrani povezavo na widget, za katerega rišemo.
		 * @param widget widget, na katerega rišemo
		 * @param shape tip izrisovanja kontrolnih točk
		 */
		HoverPoints( QWidget *widget, PointShape shape );

		/**
		 * Filtrira evente za paint in click.
		 * @param object
		 * @param event
		 * @return
		 */
		bool eventFilter( QObject *object, QEvent *event );

		/**
		 * Nariše točke grafa.
		 */
		void paintPoints();

		/**
		 * Vrne bounding rect.
		 * @return
		 */
		inline QRectF boundingRect() const;
		/**
		 * Nastavi obkrožajoči kvadrat.
		 * @param boundingRect
		 */
		void setBoundingRect( const QRectF &boundingRect ) {
			m_bounds = boundingRect;
		}

		/**
		 * Vrne točke grafa
		 * @return
		 */
		QPolygonF points() const {
			return m_points;
		}
		/**
		 * Nastavi točke grafa
		 * @param points
		 */
		void setPoints( const QPolygonF &points );

		/**
		 * Vrne velikost točk
		 * @return
		 */
		QSizeF pointSize() const {
			return m_pointSize;
		}
		/**
		 * Nastavi velikost točk na podano velikost
		 * @param size podana velikost
		 */
		void setPointSize( const QSizeF &size ) {
			m_pointSize = size;
		}

		/**
		 * Vrne tip sortiranja točk
		 * @return tip sortiranja točk
		 */
		SortType sortType() const {
			return m_sortType;
		}
		/**
		 * Nastavi tip sortiranja točk
		 * @param sortType tip sortiranja točk
		 */
		void setSortType( SortType sortType ) {
			m_sortType = sortType;
		}

		/**
		 * Vrne tip izrisovanja povezav med kontrolnimi točkami
		 * @return tip izrisovanja povezav med kontrolnimi točkami
		 */
		ConnectionType connectionType() const {
			return m_connectionType;
		}
		/**
		 * Nastavi tip izrisovanja povezav med kontrolnimi točkami
		 * @param connectionType tip izrisovanja povezav med kontrolnimi točkami
		 */
		void setConnectionType( ConnectionType connectionType ) {
			m_connectionType = connectionType;
		}

		/**
		 * Nastavi barvo in debelino izrisa povezav med kontrolnimi točkami
		 * @param pen barva in debelina izrisa povezav med kontrolnimi točkami
		 */
		void setConnectionPen( const QPen &pen ) {
			m_connectionPen = pen;
		}
		/**
		 * Nastavi barvo in debelino izrisa kontrolnih točk
		 * @param pen barva in debelina izrisa kontrolnih točk
		 */
		void setShapePen( const QPen &pen ) {
			m_pointPen = pen;
		}
		/**
		 * Nastavi polnilo izrisa kontrolnih točk
		 * @param brush polnilo izrisa kontrolnih točk
		 */
		void setShapeBrush( const QBrush &brush ) {
			m_pointBrush = brush;
		}
		/**
		 * Nastavi zabitje točke na danem indeksu na dani tip
		 * @param pos dan indeks
		 * @param lock dan tip
		 */
		void setPointLock( int pos, LockType lock ) {
			m_locks[ pos ] = lock;
		}
		/**
		 * Nastavi možnost urejanja točk, da uporabnik klika in spreminja točke
		 * @param editable kretnica
		 */
		void setEditable( bool editable ) {
			m_editable = editable;
		}
		/**
		 * Vrne, ali uporabnik sme interaktivno spreminjati kontrolne točke
		 * @return
		 */
		bool editable() const {
			return m_editable;
		}

	public slots:
		/**
		 * Nastavi omogočenost widgeta: ali filtrira evente
		 * @param enabled omogočenost
		 */
		void setEnabled( bool enabled );
		/**
		 * Onemogoči widget, da ne filtrira eventov
		 * @param disabled onemogočenost
		 */
		void setDisabled( bool disabled ) {
			setEnabled( !disabled );
		}

	signals:
		/**
		 * Poslan, ko uporabnik spremeni kontrolne točke
		 * @param points spremenjene kontrolne točke
		 */
		void pointsChanged( const QPolygonF &points );
		/**
		 * Poslan, ko se spremeni položaj miške nad kontrolo:
		 * uporabljen za barvanje po G in W indeksima.
		 * @param point kliknjena točka na widgetu
		 */
		void hoverChanged(const QPointF &point);

	public:
		/**
		 * Povzroči izstrelitev signala, da so se spremenile točke
		 */
		void firePointChange();
		/**
		 * Vrne indeks trenutno urejane točke (tisto, ki držimo za spreminjanje)
		 * @return držana točka
		 */
		int getCurrentIndex();

	private:
		/**
		 * Vrne obkrožajoči kvader kontrolne točke, odvisen od nastavljene
		 * velikosti izrisa kroga/kvadrata na njenem mestu.
		 * @param i indeks kontrolne točke
		 * @return položaj in širina/višina od tam, koliko se razteza obkrožajoči kvader
		 */
		inline QRectF pointBoundingRect( int i ) const;
		/**
		 * Premakne točko na podanem indeksu na novo pozicijo
		 * @param i podan indeks točke
		 * @param newPos nova točka
		 * @param emitChange ali naj pri tem sproži signal, da se je točka spremenila
		 */
		inline void movePoint( int i, const QPointF &newPos, bool emitChange = true );

		/**
		 * Kontrola, na katero naj rišem: lovim njene paint evente.
		 */
		QWidget *m_widget;

		/**
		 * Kontrolne točke grafa: seznam float točk (x,y).
		 */
		QPolygonF m_points;
		/**
		 * Obkrožajoči kvader vseh narisanih kontrolnih točk.
		 */
		QRectF m_bounds;
		/**
		 * Oblika izrisa kontrolne točke.
		 */
		PointShape m_shape;
		/**
		 * Način sortiranja točk.
		 */
		SortType m_sortType;
		/**
		 * Tip povezav med kontrolnimi točkami.
		 */
		ConnectionType m_connectionType;

		/**
		 * Seznam pribitev točk po indeksih.
		 */
		QVector<uint> m_locks;

		/**
		 * Velikost risane točke
		 */
		QSizeF m_pointSize;
		/**
		 * Indeks trenutno izbrane točke
		 */
		int m_currentIndex;
		/**
		 * Ali je graf možno urejati
		 */
		bool m_editable;
		/**
		 * Ali smem loviti evente.
		 */
		bool m_enabled;

		/**
		 * Barva, debelina za izris obrobe kontrolnih točk
		 */
		QPen m_pointPen;
		/**
		 * Polnilo za notranjost izrisa kontrolnih točk.
		 */
		QBrush m_pointBrush;
		/**
		 * Barva, debelina za izris povezav med kontrolnimi točkami.
		 */
		QPen m_connectionPen;
	};

	inline QRectF HoverPoints::pointBoundingRect( int i ) const {
		QPointF p = m_points.at( i );
		double w = m_pointSize.width();
		double h = m_pointSize.height();
		double x = p.x() - w / 2;
		double y = p.y() - h / 2;
		return QRectF( x, y, w, h );
	}

	inline QRectF HoverPoints::boundingRect() const {
		if ( m_bounds.isEmpty() )
			return m_widget->rect();
		else
			return m_bounds;
	}
}
#endif // HOVERPOINTS_H
