/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "ui_mainwindow.h"
#include <vector>

namespace ecosystem {
	namespace render{
		class EcosystemRender;
	}
}

namespace gui {
	/*!
	 * \namespace gui
	 * \brief skrbi za nadzor aplikacije, poveže knjižnici drevo in ecosystem
	 *
	 * Vsa koda za GUI interakcijo se nahaja v tem imenskem prostoru.
	 * MainWindow vsebuje enaga ali več OglWidget-ov, ki vsebujejo OpenGL-kontekste,
	 * na katere se uporablja geometrija.
	 *
	 * Tudi modelov je lahko več, to so iz razreda ecosystem::render::EcosystemRender.
	 * OglWidget je pogled, EcosystemRender model. Model lahko ima več pogledov,
	 * pogled pa lahko zamenja ogledovan model. Povezava je N:N.
	 *
	 * V MainWindow je vsebovan OglWidget, v tem je EcosystemRender, ki nariše
	 * geometrijo. OglWidget lahko izriše 2 načina: prvi je ekosistem, drugi pa
	 * načrtovanje enega drevesa.
	 *
	 * MainWindow za obdelovanje uporabnikovih akcij uporablja razrede v imenskem
	 * prostoru actionhandling: DesignHandler skrbi za načrtovanje grafov,
	 * FileHandler za delo z datotekami, NavigationHandler za postavljanje kamere,
	 * TimeKeeper za poganjanje timerjev animacije, ViewHandler za spreminjanje
	 * načinov pogleda na drevesa (ne pa tudi kamere).
	 *
	 * Dialogi: About prikaže osnovne podatke o programu, AdvancedTreeProperties ureja
	 * parametre drevesa, DistributionDesigner prikaže 2 grafa za G,W parametre
	 * dreves, EcosystemStatistics prikaže statistiko o rastlinah v ekosistemu,
	 * NavigationPanel prikaže ploščo za nastavljanje kamere, PixmapExportDialog
	 * nastavlja lastnosti izvažane zaslonske slike.
	 *
	 * WidgetGraph je za risanje in urejanje grafa, ki vsebuje HoverPoints.
	 *
	 * @author Ales Zamuda
	 */


	class OglWidget;
	namespace actionhandling {
		class FileHandler;
		class TimeKeeper;
		class DesignHandler;
		class ViewHandler;
		class NavigationHandler;
	}

	/**
	 * Razred za prikaz glavnega okna aplikacije.
	 * Vsebuje enaga ali več kontrol OglWidget, ki izrisujejo geometrijo.
	 *
	 * OglWidget prejme kot kazalec notri že obstoječi model, če ne pa
	 * ustvari nov model in o tem obvesti MainWindow.
	 *
	 * Ostali member objekti skrbijo za handlanje uporabnikovih interakcij
	 * z uporabnikom.
	 *
	 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
	 */
	class MainWindow : public QMainWindow, private Ui::MainWindow {
		Q_OBJECT

	public:
		/**
		 * Ustvari nov objekt.
		 * Pred tem se je najverjetneje v aplikaciji vzpostavilo prevajanje
		 * uporabniškega jezika vmesnika in pokazal QSplashScreen.
		 * @param parent objekt, ki me zbriše
		 */
		MainWindow( QWidget *parent = 0 );
		/**
		 * Uniči aplikacijo
		 */
		~MainWindow();

	public slots:
		/**
		 * OglWidget sporoči, da je kreiral nov model, mi ga pomnimo,
		 * da nastavljamo parametre z kasnejšimi dialogi.
		 *
		 * @param model nov model
		 */
		void bindNewModel(ecosystem::render::EcosystemRender *model);

	protected slots:
		///TODO: odstrani, te še zaenkrat sam handlam:
		/**
		 * Sprožen za nastavljanje območja rasti rastlin.
		 */
		void on_actionSet_plant_area_triggered();
		/**
		 * Prikaže dialog za urejanje semen random generatorja.
		 */
		void on_actionEdit_random_generator_seeds_triggered();
		/**
		 * Prikaže dialog za statistiko
		 */
		void on_actionShow_statistics_triggered();
		/**
		 * Prikaže dialog za navodila
		 */
		void on_actionUsage_triggered();
		/**
		 * Prikaže dialog o aplikaciji
		 */
		void on_actionAbout_triggered();
		/**
		 * Doda nov pogled na trenutni model (doda OglWidget)
		 */
		void on_actionAdd_view_triggered();
		/**
		 * Preklopi na naslednji pogled, za trenutni model
		 */
		void cycleView();
		/**
		 * ActionHandling kontrole poveže z novim OglWidget
		 * @param oglWidget novi OglWidget
		 */
		void newGlContextInitialized(OglWidget *oglWidget);

		// void on_actionQuit_triggered(){} // NE RABI CONNECTA

	private:
		/** Vsebuje model za aplikacijo: trenutno drevo, ki ga naj riše ali pa riše celoten ekositem */
		ecosystem::render::EcosystemRender *currentModel;
		/**
		 * Seznam modelov v aplikaciji: N:N z OglWidget
		 */
		std::vector<ecosystem::render::EcosystemRender*> models;
		/**
		 * Indeks trenutno izbranega modela
		 */
		int currentModelNr;

		/**
		 * Trenutno izbrani pogled na trenutni model
		 */
		gui::OglWidget *oglWidget;
		/**
		 * Objekt za obravnavno akcij dela z datotekami
		 */
		actionhandling::FileHandler *fileHandler;
		/**
		 * Objekt za obravnavno akcij dela z časovniki
		 */
		actionhandling::TimeKeeper *timeKeeper;
		/**
		 * Objekt za obravnavno akcij dela z načrtovanjem parametrov drevesa
		 */
		actionhandling::DesignHandler *designHandler;
		/**
		 * Objekt za obravnavno akcij spreminjanja izgleda
		 */
		actionhandling::ViewHandler *viewHandler;
		/**
		 * Objekt za obravnavno akcij spreminjanja kamere
		 */
		actionhandling::NavigationHandler *navigationHandler;
	};
}
#endif
