/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef GUIOGLWIDGET_H
#define GUIOGLWIDGET_H

#include <QGLWidget>
class QMouseEvent;
namespace ecosystem {
namespace render{
	class EcosystemRender;
}
}
namespace gui {
	/**
	 * OpenGL enabled widget, GUI functionallity.
	 * Omogoča kontrolo določenega modela, ki je lahko deljen z večimi OglWidgeti, da
	 * bi ustvarili različne poglede. Model lahko med izvajanjem tudi zamenjamo,
	 * tako da imamo lahko en widget izmenjaje tudi za več modelov.
	 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
	*/
	class OglWidget : public QGLWidget {
		Q_OBJECT
	public:
		/**
		 * Kreira widget in shrani povezavo na model.
		 *
		 * TODO: naj model pošilja signale, če kaj rabi (teksture,...)
		 *
		 * @param parent
		 * @param ecosystemRender
		 */
		OglWidget(QWidget *parent, ecosystem::render::EcosystemRender *ecosystemRender);
		/**
		 * Odstrani objekt. Modela ne izbriše, saj si ga lahko delijo še drugi
		 * pogledi (OglWidget-i).
		 */
		virtual ~OglWidget();

		/**
		 * Inicializira vse, kar mora imeti OpenGL Context, da riše model,
		 * neodvisno od tega, v kakšnem stanju se nahaja model,
		 * lahko da že med tekom simulacije ali pa pač ob zagonu aplikacije.
		 */
		void initializeGL();
		/**
		 * Klican ob upodobitvi widget-a. Pred klicem se zgodi, da
		 * ta kontekst postane trenuten, po koncu pa se pokliče swapbuffer.
		 */
		void paintGL();
		/**
		 * Projekcijsko matriko prilagodi trenutnemu oknu.
		 * @param width širina okna za OpenGL kontekst
		 * @param height višina okna za OpenGL kontekst
		 */
		void resizeGL( int width, int height );

	signals:
		/**
		 * Poslan, če OglWidget sam kreira model (ob konstrukciji je dobil model 0)
		 * @param value kreiran model
		 */
		void newModelCreated(ecosystem::render::EcosystemRender *);
		/**
		 * Klican po tem, ko je že inicializiran OglWidget,
		 * pa tudi morebitni newModelCreated že signaliziran.
		 *
		 * @param
		 */
		void newGlContextInitialized(OglWidget *);
		/**
		 * S klikom na ta view postavimo ta view trenuten v MainWindow za navigacijo,
		 * da se signali dispachajo sem. Sprožen, ko postane fokusiran.
		 */
		void grabCamera();

	public slots:
		/**
		 * Povzroči preklop prikazovanega modela v tem OglWidget-u na podan model.
		 * @param model podan model
		 */
		void switchToModel(ecosystem::render::EcosystemRender *model);
		/**
		 * Window event za premik miške
		 * @param event event za premik miške
		 */
		void mouseMoveEvent ( QMouseEvent * event );
		/**
		 * Window event za pritisk gumba miške
		 * @param event event za premik miške
		 */
		void mousePressEvent ( QMouseEvent * event );
		/**
		 * Window event za spust gumba miške
		 * @param event event za premik miške
		 */
		void mouseReleaseEvent ( QMouseEvent * event );

	private:
		/**
		 * Nastavi pozicije luči.
		 */
		void setLightPosition();
		/**
		 * Preveri, ali ta dobljen OpenGL kontekst podpira
		 * multiteksturiranje in compiled vertex arrays.
		 */
		void checkExtensions();

		/**
		 * Kontekst podpira multiteksturiranje.
		 */
		bool supportMultitexture;
		/**
		 * Kontekst podpira compiled vertex arrays.
		 */
		bool supportCompiledVertexArrays;

	private:
		/** Vsebuje model za izrisovanje.
		 * Model se odloči, ali bo izrisal eno drevo (razred Drevo) ali pa
		 * bo izrisal ekosistem (Terrain in Drevo).*/
		ecosystem::render::EcosystemRender *currentModel;

		/**
		 * X koordinata začetka vlečenja miške pri sukanju drevesa
		 */
		int s_x;
		/**
		 * Y koordinata začetka vlečenja miške pri sukanju drevesa
		 */
		int s_y;
		/**
		 * Y koordinata začetka vlečenja miške pri premikanju drevesa ven/notri
		 */
		int t_z;
		/**
		 * Trenutno z miško premikam Drevo.
		 */
		bool actionMove;
		/**
		 * Trenutno z miško sučem Drevo.
		 */
		bool actionRotate;
		/**
		 * Natančnost miške pri spreminjanju pogleda.
		 */
		float mouseSensivity;
	};
}
#endif
